/**
 * 
 */
package es.postigo.dao;

import java.util.List;

import es.postigo.dto.Inventario;
import es.postigo.dto.InventarioPk;
import es.postigo.exceptions.ApplicationException;
import es.postigo.exceptions.InventarioDaoException;

/**
 * @author informatica4
 *
 */
public interface InventarioDao {
	/**
	 * Method 'insert'
	 * 
	 * @param dto
	 * @return InventarioPk
	 * @throws InventarioDaoException 
	 */
	public InventarioPk insert(Inventario dto) throws InventarioDaoException;

	/** 
	 * Updates a single row in the Inventario table.
	 */
	public void update(InventarioPk pk, Inventario dto) throws InventarioDaoException, ApplicationException;
	
	/** 
	 * Updates a single row in the Inventario table.
	 */
	public void incrementarInventario(InventarioPk pk, Inventario dto) throws InventarioDaoException, ApplicationException;
	
	/** 
	 * Updates a single row in the Inventario table.
	 */
	public void incrementarInventario(InventarioPk pk, Inventario dto, String pStrCodUbicacionOrigen) throws InventarioDaoException, ApplicationException;

	
	/** 
	 * Updates a single row in the Inventario table.
	 */
	public void decrementarInventario(InventarioPk pk, Inventario dto) throws InventarioDaoException, ApplicationException;
	
	/** 
	 * Updates a single row in the Inventario table.
	 */
	public void decrementarInventario(InventarioPk pk, Inventario dto, String pStrCodUbicacionDestino) throws InventarioDaoException, ApplicationException;

	/** 
	 * Deletes a single row in the Inventario table.
	 */
	public void delete(String codUbicacion, String codProducto) throws InventarioDaoException;

	/** 
	 * Returns all rows from the Inventario table that match the criteria 'CODIGO = :codigo'.
	 */
	public Inventario findByPrimaryKey(String codUbicacion, String codProducto) throws InventarioDaoException;
	
	public Inventario findByCodigosEAN(String codEanUbicacion, String codEanProducto) throws InventarioDaoException;

	/** 
	 * Returns all rows from the Inventario table that match the criteria ''.
	 */
	public List<Inventario> findAll() throws InventarioDaoException;

	/** 
	 * Returns all rows from the Inventario table that match the criteria 'CODIGO = :codigo'.
	 */
	public List<Inventario> findWhereCodUbicacionEquals(String codigo) throws InventarioDaoException;

	/** 
	 * Returns all rows from the Inventario table that match the criteria 'Descripcion = :descripcion'.
	 */
	public List<Inventario> findWhereCodEanProductoEquals(String codigo) throws InventarioDaoException;
	
	/** 
	 * Returns all rows from the Inventario table that match the criteria 'CODIGO = :codigo'.
	 */
	public List<Inventario> findWhereCodEanUbicacionEquals(String codigo) throws InventarioDaoException;

	/** 
	 * Returns all rows from the Inventario table that match the criteria 'Descripcion = :descripcion'.
	 */
	public List<Inventario> findWhereCodProductoEquals(String codigo) throws InventarioDaoException;
	
	/** 
	 * Returns all rows from the Inventario table that match the criteria '"+ pstrSQL +"'.
	 */
	public List<Inventario> findByDynamicWhere(String pstrSQL) throws InventarioDaoException;

	/** 
	 * Returns the rows from the Inventario table that matches the specified primary-key value.
	 */
	public Inventario findByPrimaryKey(InventarioPk pk) throws InventarioDaoException;
	
	/** 
	 * Returns all rows from the Inventario table that match the criteria '"+ pstrSQL +"'.
	 */
	public List<Inventario> findByDynamicSelect(String pstrSQL) throws InventarioDaoException;
}
