/**
 * 
 */
package es.postigo.dao;

import java.util.List;

import es.postigo.dto.Producto;
import es.postigo.dto.ProductoPk;
import es.postigo.exceptions.ApplicationException;
import es.postigo.exceptions.ProductoDaoException;

/**
 * @author informatica4
 *
 */
public interface ProductoDao {

	/**
	 * Method 'insert'
	 * 
	 * @param dto
	 * @return ProductoPk
	 * @throws ProductoDaoException 
	 */
	public ProductoPk insert(Producto dto) throws ProductoDaoException;
	
	public ProductoPk insert(Producto dto, String pStrProducto) throws ProductoDaoException;

	/** 
	 * Updates a single row in the Producto table.
	 */
	public void update(ProductoPk pk, Producto dto) throws ProductoDaoException, ApplicationException;

	/** 
	 * Deletes a single row in the Producto table.
	 */
	public void delete(String codigo) throws ProductoDaoException;

	/** 
	 * Returns all rows from the Producto table that match the criteria 'CODIGO = :codigo'.
	 */
	public Producto findByPrimaryKey(String codigo) throws ProductoDaoException;
	
	/** 
	 * Returns all rows from the Producto table that match the criteria 'CODIGOEAN = :codigoEAN'.
	 */
	public Producto findByCodigoEAN(String codigoEAN) throws ProductoDaoException;

	/** 
	 * Returns all rows from the Producto table that match the criteria ''.
	 */
	public List<Producto> findAll() throws ProductoDaoException;

	/** 
	 * Returns all rows from the Producto table that match the criteria 'CODIGO = :codigo'.
	 */
	public List<Producto> findWhereCodigoEquals(String codigo) throws ProductoDaoException;

	/** 
	 * Returns all rows from the Producto table that match the criteria 'Descripcion = :descripcion'.
	 */
	public List<Producto> findWhereDescripcionEquals(String descripcion) throws ProductoDaoException;
	
	/** 
	 * Returns all rows from the Producto table that match the criteria '"+ pstrSQL +"'.
	 */
	public List<Producto> findByDynamicWhere(String pstrSQL) throws ProductoDaoException;

	/** 
	 * Returns the rows from the Producto table that matches the specified primary-key value.
	 */
	public Producto findByPrimaryKey(ProductoPk pk) throws ProductoDaoException;
	
	/** 
	 * Returns all rows from the Producto table that match the criteria '"+ pstrSQL +"'.
	 */
	public List<Producto> findByDynamicSelect(String pstrSQL) throws ProductoDaoException;
	
	/** 
	 * Returns Last inserted element.
	 */
	public String obtenerUltimoElementoInsertado() throws ProductoDaoException;
}
