/**
 * 
 */
package es.postigo.dao.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import es.postigo.dao.InventarioDao;
import es.postigo.delegate.ProductoDelegate;
import es.postigo.dto.Inventario;
import es.postigo.dto.InventarioPk;
import es.postigo.dto.Producto;
import es.postigo.exceptions.InventarioDaoException;
import es.postigo.jdbc.ResourceManager;
import es.postigo.utility.ConexionBD;
import es.postigo.utility.EnviarMail;

/**
 * @author informatica4
 *
 */
public class InventarioDaoImpl extends AbstractDataAccessObject implements InventarioDao
{
	/** 
	 * The factory class for this DAO has two versions of the create() method - one that
takes no arguments and one that takes a Connection argument. If the Connection version
is chosen then the connection will be stored in this attribute and will be used by all
calls to this DAO, otherwise a new Connection will be allocated for each operation.
	 */
	protected java.sql.Connection userConn;

	/** 
	 * All finder methods in this class use this SELECT constant to build their queries
	 */
	protected final String SQL_SELECT = "SELECT COD_UBICACION, COD_PRODUCTO, CANTIDAD FROM " + getTableName() + "";

	/** 
	 * SQL INSERT statement for this table
	 */
	protected final String SQL_INSERT = "INSERT INTO " + getTableName() + " ( COD_UBICACION, COD_PRODUCTO, CANTIDAD, CODEAN_UBICACION, CODEAN_PRODUCTO ) VALUES ( ?, ?, ?, ?, ? )";

	/** 
	 * SQL UPDATE statement for this table
	 */
	protected final String SQL_UPDATE = "UPDATE " + getTableName() + " SET COD_UBICACION = ?, COD_PRODUCTO = ?, CODEAN_UBICACION = ?, CODEAN_PRODUCTO = ?, CANTIDAD = ? WHERE COD_UBICACION = ? AND COD_PRODUCTO = ?";
	
	/** 
	 * SQL UPDATE statement for this table
	 */
	protected final String SQL_ENTRADAALMACEN = "UPDATE " + getTableName() + " SET COD_UBICACION = ?, COD_PRODUCTO = ?, CANTIDAD = CANTIDAD + ? WHERE COD_UBICACION = ? AND COD_PRODUCTO = ?";

	/** 
	 * SQL UPDATE statement for this table
	 */
	protected final String SQL_SALIDAALMACEN = "UPDATE " + getTableName() + " SET COD_UBICACION = ?, COD_PRODUCTO = ?, CANTIDAD = CANTIDAD - ? WHERE COD_UBICACION = ? AND COD_PRODUCTO = ?";


	/** 
	 * SQL DELETE statement for this table
	 */
	protected final String SQL_DELETE = "DELETE FROM " + getTableName() + " WHERE COD_UBICACION = ? AND COD_PRODUCTO = ?";

	/** 
	 * Index of column COD_UBICACION
	 */
	protected static final int COLUMN_COD_UBICACION = 1;

	/** 
	 * Index of column COD_PRODUCTO
	 */
	protected static final int COLUMN_COD_PRODUCTO = 2;
	
	/** 
	 * Index of column CANTIDAD
	 */
	protected static final int COLUMN_CANTIDAD = 3;
	
	/** 
	 * Index of column COD_UBICACION
	 */
	protected static final int COLUMN_COD_EAN_UBICACION = 4;

	/** 
	 * Index of column COD_PRODUCTO
	 */
	protected static final int COLUMN_COD_EAN_PRODUCTO = 5;
	
	/** 
	 * Number of columns
	 */
	protected static final int NUMBER_OF_COLUMNS = 5;

	/** 
	 * Index of primary-key column COD_UBICACION and COD_PRODUCTO
	 */
	protected static final int PK_COLUMN_COD_UBICACION = 1;
	
	protected static final int PK_COLUMN_COD_PRODUCTO = 2;

	/** 
	 * Inserts a new row in the INVENTARIO table.
	 */
	public InventarioPk insert(Inventario dto) throws InventarioDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			stmt = conn.prepareStatement( SQL_INSERT );
			if (dto.getCodUbicacion() != null) {
				stmt.setString( COLUMN_COD_UBICACION, dto.getCodUbicacion() );
			} else {
				stmt.setNull(COLUMN_COD_UBICACION, Types.INTEGER);
			}
			
			if (dto.getCodProducto() != null) {
				stmt.setString( COLUMN_COD_PRODUCTO, dto.getCodProducto() );
			} else {
				stmt.setNull(COLUMN_COD_PRODUCTO, Types.INTEGER);
			}
			
			if (dto.getCodEanUbicacion() != null) {
				stmt.setString( COLUMN_COD_EAN_UBICACION, dto.getCodEanUbicacion() );
			} else {
				stmt.setNull(COLUMN_COD_EAN_UBICACION, Types.INTEGER);
			}
			
			if (dto.getCodEanProducto() != null) {
				stmt.setString( COLUMN_COD_EAN_PRODUCTO, dto.getCodEanProducto() );
			} else {
				stmt.setNull(COLUMN_COD_EAN_PRODUCTO, Types.INTEGER);
			}
		
			stmt.setString( COLUMN_CANTIDAD, String.valueOf(dto.getCantidad()));
			
			/**Insertamos en Movimientos para tener un historico*/
			int result = stmt.executeUpdate("INSERT INTO MOVIMIENTOS (UBI_ORIGEN, UBI_DESTINO, COD_PRODUCTO, CANTIDAD, FECHA) VALUES ( '999000000000', '" + dto.getCodUbicacion() + "', '" + dto.getCodProducto() + "', '" + Integer.valueOf(dto.getCantidad()) + "', Now())");
			
			System.out.println( "Executing " + SQL_INSERT + " with DTO: " + dto );
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
			return dto.createPk();
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Updates a single row in the INVENTARIO table.
	 */
	public void update(InventarioPk pk, Inventario dto) throws InventarioDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			System.out.println( "Executing " + SQL_UPDATE + " with DTO: " + dto );
			stmt = conn.prepareStatement( SQL_UPDATE );
			if (dto.getCodUbicacion() != null) {
				stmt.setString( COLUMN_COD_UBICACION, dto.getCodUbicacion() );
			} else {
				stmt.setNull(COLUMN_COD_UBICACION, Types.INTEGER);
			}
			if (dto.getCodProducto() != null) {
				stmt.setString( COLUMN_COD_PRODUCTO, dto.getCodProducto() );
			} else {
				stmt.setNull(COLUMN_COD_PRODUCTO, Types.INTEGER);
			}
		
			stmt.setString( COLUMN_CANTIDAD, String.valueOf(dto.getCantidad()));
			if (pk.getCodUbicacion() != null) {
				stmt.setString( 4, pk.getCodUbicacion() );
			} else {
				stmt.setNull(4, Types.INTEGER);
			}
			if (pk.getCodProducto() != null) {
				stmt.setString( 5, pk.getCodProducto() );
			} else {
				stmt.setNull(5, Types.INTEGER);
			}
		
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Deletes a single row in the INVENTARIO table.
	 */
	public void delete(String pStrCodUbicacion, String pStrCodProducto) throws InventarioDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			System.out.println( "Executing " + SQL_DELETE + " with PK: " + pStrCodUbicacion + ", " + pStrCodProducto );
			stmt = conn.prepareStatement( SQL_DELETE );
			if (pStrCodUbicacion != null) {
				stmt.setString( 1, pStrCodUbicacion );
			} else {
				stmt.setNull(1, Types.INTEGER);
			}
			if (pStrCodProducto != null) {
				stmt.setString( 2, pStrCodProducto );
			} else {
				stmt.setNull(2, Types.INTEGER);
			}
		
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Returns the rows from the INVENTARIO table that matches the specified primary-key value.
	 */
	public Inventario findByPrimaryKey(InventarioPk pk) throws InventarioDaoException
	{
		return findByPrimaryKey( pk );
	}

	/** 
	 * Returns all rows from the INVENTARIO table.
	 */
	public List<Inventario> findAll() throws InventarioDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " ORDER BY COD_UBICACION", null );
	}

	/** 
	 * Returns all rows from the INVENTARIO table that match the criteria 'CODIGO = :codigo'.
	 */
	public Inventario findByPrimaryKey(Integer codUbicacion, Integer codProducto) throws InventarioDaoException
	{
		List<Inventario> ret = findByDynamicSelect( SQL_SELECT + " WHERE COD_UBICACION = ? AND COD_PRODUCTO = ?", new Object[] { codUbicacion, codProducto } );
		return ret.size()==0 ? null : ret.get(0);
	}
		
	/** 
	 * Returns all rows from the INVENTARIO table that match the criteria 'CODIGO = :codigo'.
	 */
	public List<Inventario> findWhereCodUbicacionEquals(String codUbicacion) throws InventarioDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE COD_UBICACION = ? ORDER BY COD_UBICACION", new Object[] { codUbicacion } );
	}
	
	public List<Inventario> findWhereCodProductoEquals(String codProducto) throws InventarioDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE COD_PRODUCTO = ? ORDER BY COD_PRODUCTO", new Object[] { codProducto } );
	}
	
	/** 
	 * Returns all rows from the INVENTARIO table that match the criteria 'CODIGO = :codigo'.
	 */
	public List<Inventario> findWhereCodEanUbicacionEquals(String codEanUbicacion) throws InventarioDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE CODEAN_UBICACION = ? ORDER BY CODEAN_UBICACION", new Object[] { codEanUbicacion } );
	}
	
	public List<Inventario> findWhereCodEanProductoEquals(String codEanProducto) throws InventarioDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE CODEAN_PRODUCTO = ? ORDER BY CODEAN_PRODUCTO", new Object[] { codEanProducto } );
	}

	/** 
	 * Returns all rows from the INVENTARIO table that match the criteria 'CANTIDAD = :cantidad'.
	 */
	public List<Inventario> findWhereCantidadEquals(String cantidad) throws InventarioDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE CANTIDAD = ? ORDER BY CANTIDAD", new Object[] { cantidad } );
	}

	/**
	 * Method 'InventarioDaoImpl'
	 * 
	 */
	public InventarioDaoImpl()
	{
	}

	/**
	 * Method 'InventarioDaoImpl'
	 * 
	 * @param userConn
	 */
	public InventarioDaoImpl(final java.sql.Connection userConn)
	{
		this.userConn = userConn;
	}

	/**
	 * Method 'getTableName'
	 * 
	 * @return String
	 */
	public String getTableName()
	{
		return "INVENTARIO";
	}

	/** 
	 * Fetches a single row from the result set
	 */
	protected Inventario fetchSingleResult(ResultSet rs) throws SQLException
	{
		if (rs.next()) {
			Inventario dto = new Inventario();
			populateDto( dto, rs);
			return dto;
		} else {
			return null;
		}
		
	}

	/** 
	 * Fetches multiple rows from the result set
	 */
	protected List<Inventario> fetchMultiResults(ResultSet rs) throws SQLException
	{
		Collection resultList = new ArrayList();
		while (rs.next()) {
			Inventario dto = new Inventario();
			populateDto( dto, rs);
			resultList.add( dto );
		}
		return (List<Inventario>) resultList;
	}

	/** 
	 * Populates a DTO with data from a ResultSet
	 */
	protected void populateDto(Inventario dto, ResultSet rs) throws SQLException
	{
		dto.setCodUbicacion( rs.getString(COLUMN_COD_UBICACION));
		dto.setCodProducto( rs.getString(COLUMN_COD_PRODUCTO));
		dto.setCantidad( rs.getString(COLUMN_CANTIDAD));
	}

	/** 
	 * Returns all rows from the INVENTARIO table that match the specified arbitrary SQL statement
	 */
	public List<Inventario> findByDynamicSelect(String sql, Object[] sqlParams) throws InventarioDaoException
	{
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			// construct the SQL statement
			final String SQL = sql;
		
		
			System.out.println( "Executing " + SQL );
			// prepare statement
			stmt = conn.prepareStatement( SQL );
		
			// bind parameters
			for (int i=0; sqlParams!=null && i<sqlParams.length; i++ ) {
				stmt.setObject( i+1, sqlParams[i] );
			}
		
		
			rs = stmt.executeQuery();
		
			// fetch the results
			return fetchMultiResults(rs);
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(rs);
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Returns all rows from the INVENTARIO table that match the specified arbitrary SQL statement
	 */
	public List<Inventario> findByDynamicWhere(String sql, Object[] sqlParams) throws InventarioDaoException
	{
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			// construct the SQL statement
			final String SQL = SQL_SELECT + " WHERE " + sql;
		
		
			System.out.println( "Executing " + SQL );
			// prepare statement
			stmt = conn.prepareStatement( SQL );
		
			// bind parameters
			for (int i=0; sqlParams!=null && i<sqlParams.length; i++ ) {
				stmt.setObject( i+1, sqlParams[i] );
			}
		
		
			rs = stmt.executeQuery();
		
			// fetch the results
			return fetchMultiResults(rs);
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(rs);
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}
	
	public List<Inventario> findByDynamicWhere(String pstrSQL)
			throws InventarioDaoException {
		return findByDynamicWhere(pstrSQL, null);
	}

	public Inventario findByPrimaryKey(String pStrCodUbicacion, String pStrCodProducto) throws InventarioDaoException {
		Inventario inv = null;
		Connection con;
		try {
			con = ConexionBD.obtieneConexion();
			Statement s = con.createStatement();
			String strSQL = "SELECT * FROM INVENTARIO WHERE COD_UBICACION = '" + pStrCodUbicacion + "' AND COD_PRODUCTO = '" + pStrCodProducto + "'";
			ResultSet rs = s.executeQuery(strSQL);
			if(rs.next()){
				System.out.println("ok");
				inv = new Inventario();
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		return inv;
		//return findByPrimaryKey( codUbicacion, codProducto );
	}
	
	public Inventario findByCodigosEAN(String pStrCodEanUbicacion, String pStrCodEanProducto) throws InventarioDaoException {
		Inventario inv = null;
		Connection con;
		try {
			con = ConexionBD.obtieneConexion();
			Statement s = con.createStatement();
			String strSQL = "SELECT * FROM INVENTARIO WHERE CODEAN_UBICACION = '" + pStrCodEanUbicacion + "' AND CODEAN_PRODUCTO = '" + pStrCodEanProducto + "'";
			ResultSet rs = s.executeQuery(strSQL);
			if(rs.next()){
				System.out.println("ok");
				inv = new Inventario();
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		return inv;
		//return findByPrimaryKey( codUbicacion, codProducto );
	}

	public List<Inventario> findByDynamicSelect(String pstrSQL)
			throws InventarioDaoException {
		return findByDynamicSelect(pstrSQL, null);
	}
	
	/** 
	 * Updates a single row in the INVENTARIO table.
	 */
	public void incrementarInventario(InventarioPk pk, Inventario dto) throws InventarioDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			/**Insertamos en Movimientos para tener un historico*/
			System.out.println("INSERT INTO MOVIMIENTOS (UBI_ORIGEN, UBI_DESTINO, COD_PRODUCTO, CANTIDAD, FECHA) VALUES ( '999000000000', '" + dto.getCodUbicacion() + "', '" + dto.getCodProducto() + "', '" + Integer.valueOf(dto.getCantidad()) + "', Now())");
			insertEnMovimientos( "999000000000", dto.getCodUbicacion(), dto.getCodProducto(), Integer.valueOf(dto.getCantidad()), "entrada");
			//int result = stmt.executeUpdate("INSERT INTO MOVIMIENTOS (UBI_ORIGEN, UBI_DESTINO, COD_PRODUCTO, CANTIDAD, FECHA) VALUES ( '999000000000', '" + dto.getCodUbicacion() + "', '" + dto.getCodProducto() + "', '" + Integer.valueOf(dto.getCantidad()) + "', Now())");
			
			
			System.out.println( "Executing " + SQL_ENTRADAALMACEN + " with DTO: " + dto );
			stmt = conn.prepareStatement( SQL_ENTRADAALMACEN );
			if (dto.getCodUbicacion() != null) {
				stmt.setString( COLUMN_COD_UBICACION, dto.getCodUbicacion() );
			} else {
				stmt.setNull(COLUMN_COD_UBICACION, Types.INTEGER);
			}
			if (dto.getCodProducto() != null) {
				stmt.setString( COLUMN_COD_PRODUCTO, dto.getCodProducto() );
			} else {
				stmt.setNull(COLUMN_COD_PRODUCTO, Types.INTEGER);
			}
			
			stmt.setString( COLUMN_CANTIDAD, String.valueOf(dto.getCantidad()));
			if (pk.getCodUbicacion() != null) {
				stmt.setString( 4, pk.getCodUbicacion() );
			} else {
				stmt.setNull(4, Types.INTEGER);
			}
			if (pk.getCodProducto() != null) {
				stmt.setString( 5, pk.getCodProducto() );
			} else {
				stmt.setNull(5, Types.INTEGER);
			}
		
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}
	
	public void incrementarInventario(InventarioPk pk, Inventario dto, String pStrCodUbicacionOrigen) throws InventarioDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			/**Insertamos en Movimientos para tener un historico*/
			System.out.println("INSERT INTO MOVIMIENTOS (UBI_ORIGEN, UBI_DESTINO, COD_PRODUCTO, CANTIDAD, FECHA) VALUES ( '" + pStrCodUbicacionOrigen + "', '" + dto.getCodUbicacion() + "', '" + dto.getCodProducto() + "', '" + Integer.valueOf(dto.getCantidad()) + "', Now())");
			insertEnMovimientos( pStrCodUbicacionOrigen, dto.getCodUbicacion(), dto.getCodProducto(), Integer.valueOf(dto.getCantidad()), "movimiento");
			
			System.out.println( "Executing " + SQL_ENTRADAALMACEN + " with DTO: " + dto );
			stmt = conn.prepareStatement( SQL_ENTRADAALMACEN );
			if (dto.getCodUbicacion() != null) {
				stmt.setString( COLUMN_COD_UBICACION, dto.getCodUbicacion() );
			} else {
				stmt.setNull(COLUMN_COD_UBICACION, Types.INTEGER);
			}
			if (dto.getCodProducto() != null) {
				stmt.setString( COLUMN_COD_PRODUCTO, dto.getCodProducto() );
			} else {
				stmt.setNull(COLUMN_COD_PRODUCTO, Types.INTEGER);
			}
			
			stmt.setString( COLUMN_CANTIDAD, String.valueOf(dto.getCantidad()));
			if (pk.getCodUbicacion() != null) {
				stmt.setString( 4, pk.getCodUbicacion() );
			} else {
				stmt.setNull(4, Types.INTEGER);
			}
			if (pk.getCodProducto() != null) {
				stmt.setString( 5, pk.getCodProducto() );
			} else {
				stmt.setNull(5, Types.INTEGER);
			}
		
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}
	
	/** 
	 * Updates a single row in the INVENTARIO table.
	 */
	public void decrementarInventario(InventarioPk pk, Inventario dto) throws InventarioDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			/**Insertamos en Movimientos para tener un historico*/
			insertEnMovimientos(dto.getCodUbicacion(), "999000000000", dto.getCodProducto(), Integer.valueOf(dto.getCantidad()), "salida");
			//int result = stmt.executeUpdate("INSERT INTO MOVIMIENTOS (UBI_ORIGEN, UBI_DESTINO, COD_PRODUCTO, CANTIDAD, FECHA) VALUES ( '999000000000', '" + dto.getCodUbicacion() + "', '" + dto.getCodProducto() + "', '-" + Integer.valueOf(dto.getCantidad()) + "', Now())");
			
			/**Revisar si el stock es inferior al minimo y si es inferior enviar mail*/
			Producto lObjProducto = new Producto();
			ProductoDelegate lObjProductoDelegate = new ProductoDelegate();
			lObjProducto = lObjProductoDelegate.getRegistro(dto.getCodProducto());
			
			if(lObjProducto != null){
				int lIntCantidad = 0;
				lIntCantidad = obtenerStockTotalProductoEnTodasLasUbicaciones(lObjProducto.getCodigo());
				if(lObjProducto.getStockMin() > lIntCantidad){
					EnviarMail.enviarAvisoStockMinimo(lObjProducto);
				}
			}
			/**Fin mail stock minimo*/
			
			
			System.out.println( "Executing " + SQL_SALIDAALMACEN + " with DTO: " + dto );
			stmt = conn.prepareStatement( SQL_SALIDAALMACEN );
			if (dto.getCodUbicacion() != null) {
				stmt.setString( COLUMN_COD_UBICACION, dto.getCodUbicacion() );
			} else {
				stmt.setNull(COLUMN_COD_UBICACION, Types.INTEGER);
			}
			if (dto.getCodProducto() != null) {
				stmt.setString( COLUMN_COD_PRODUCTO, dto.getCodProducto() );
			} else {
				stmt.setNull(COLUMN_COD_PRODUCTO, Types.INTEGER);
			}
		
			stmt.setString( COLUMN_CANTIDAD, String.valueOf(dto.getCantidad()));
			if (pk.getCodUbicacion() != null) {
				stmt.setString( 4, pk.getCodUbicacion() );
			} else {
				stmt.setNull(4, Types.INTEGER);
			}
			if (pk.getCodProducto() != null) {
				stmt.setString( 5, pk.getCodProducto() );
			} else {
				stmt.setNull(5, Types.INTEGER);
			}
		
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}
	
	public void decrementarInventario(InventarioPk pk, Inventario dto, String pStrCodUbicacionDestino) throws InventarioDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			/**Insertamos en Movimientos para tener un historico*/
			insertEnMovimientos(dto.getCodUbicacion(), pStrCodUbicacionDestino, dto.getCodProducto(), Integer.valueOf(dto.getCantidad()), "salida");
			
			System.out.println( "Executing " + SQL_SALIDAALMACEN + " with DTO: " + dto );
			stmt = conn.prepareStatement( SQL_SALIDAALMACEN );
			if (dto.getCodUbicacion() != null) {
				stmt.setString( COLUMN_COD_UBICACION, dto.getCodUbicacion() );
			} else {
				stmt.setNull(COLUMN_COD_UBICACION, Types.INTEGER);
			}
			if (dto.getCodProducto() != null) {
				stmt.setString( COLUMN_COD_PRODUCTO, dto.getCodProducto() );
			} else {
				stmt.setNull(COLUMN_COD_PRODUCTO, Types.INTEGER);
			}
		
			stmt.setString( COLUMN_CANTIDAD, String.valueOf(dto.getCantidad()));
			if (pk.getCodUbicacion() != null) {
				stmt.setString( 4, pk.getCodUbicacion() );
			} else {
				stmt.setNull(4, Types.INTEGER);
			}
			if (pk.getCodProducto() != null) {
				stmt.setString( 5, pk.getCodProducto() );
			} else {
				stmt.setNull(5, Types.INTEGER);
			}
		
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new InventarioDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}
	
	public void insertEnMovimientos(String pStrUbiOrigen, String pStrUbiDestino, String pStrCodProducto, int pIntCantidad, String pStrEntradaSalida) throws InventarioDaoException {
		Inventario inv = null;
		Connection con;
		try {
			con = ConexionBD.obtieneConexion();
			Statement s = con.createStatement();
			String strSQL = "";
			if("entrada".equals(pStrEntradaSalida)){
				strSQL = "INSERT INTO MOVIMIENTOS (UBI_ORIGEN, UBI_DESTINO, COD_PRODUCTO, CANTIDAD, FECHA) VALUES ( '" + pStrUbiOrigen + "', '" + pStrUbiDestino + "', '" + pStrCodProducto + "', '" + pIntCantidad + "', Now())";
			}else if("movimiento".equals(pStrEntradaSalida)){
				strSQL = "INSERT INTO MOVIMIENTOS (UBI_ORIGEN, UBI_DESTINO, COD_PRODUCTO, CANTIDAD, FECHA) VALUES ( '" + pStrUbiOrigen + "', '" + pStrUbiDestino + "', '" + pStrCodProducto + "', '" + pIntCantidad + "', Now())";
			}else {
				strSQL = "INSERT INTO MOVIMIENTOS (UBI_ORIGEN, UBI_DESTINO, COD_PRODUCTO, CANTIDAD, FECHA) VALUES ( '" + pStrUbiOrigen + "', '" + pStrUbiDestino + "', '" + pStrCodProducto + "', '-" + pIntCantidad + "', Now())";
			}
			System.out.println(strSQL);
			s.executeUpdate(strSQL);
			System.out.println("Insercin OK en historico de movimientos");			
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public int obtenerStockTotalProductoEnTodasLasUbicaciones(String pStrCodProducto){
		int lIntCantidad = 0;
		Connection con;
		try {
			con = ConexionBD.obtieneConexion();
			Statement s = con.createStatement();
			String strSQL = "SELECT * FROM INVENTARIO WHERE COD_PRODUCTO = '" + pStrCodProducto + "'";
			ResultSet rs = s.executeQuery(strSQL);
			while(rs.next()){
				lIntCantidad += rs.getInt("CANTIDAD");
				System.out.println(lIntCantidad);
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return lIntCantidad;		
	}
	
}