/**
 * 
 */
package es.postigo.dao.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import es.postigo.dao.MovimientosDao;
import es.postigo.dto.Movimientos;
import es.postigo.dto.MovimientosPk;
import es.postigo.exceptions.MovimientosDaoException;
import es.postigo.jdbc.ResourceManager;
import es.postigo.utility.ConexionBD;

/**
 * @author informatica4
 *
 */
public class MovimientosDaoImpl extends AbstractDataAccessObject implements MovimientosDao
{
	/** 
	 * The factory class for this DAO has two versions of the create() method - one that
takes no arguments and one that takes a Connection argument. If the Connection version
is chosen then the connection will be stored in this attribute and will be used by all
calls to this DAO, otherwise a new Connection will be allocated for each operation.
	 */
	protected java.sql.Connection userConn;

	/** 
	 * All finder methods in this class use this SELECT constant to build their queries
	 */
	protected final String SQL_SELECT = "SELECT UBI_ORIGEN, UBI_DESTINO, COD_PRODUCTO, CANTIDAD, FECHA FROM " + getTableName() + "";

	/** 
	 * SQL INSERT statement for this table
	 */
	protected final String SQL_INSERT = "INSERT INTO " + getTableName() + " ( UBI_ORIGEN, UBI_DESTINO, COD_PRODUCTO, CANTIDAD, FECHA ) VALUES ( ?, ?, ?, ?, ? )";

	/** 
	 * SQL UPDATE statement for this table
	 */
	protected final String SQL_UPDATE = "UPDATE " + getTableName() + " SET UBI_ORIGEN = ?, UBI_DESTINO = ?, COD_PRODUCTO = ?, CANTIDAD = ?, FECHA = ? WHERE UBI_ORIGEN = ? AND UBI_DESTINO = ? AND COD_PRODUCTO = ? AND FECHA = ?";
		
	/** 
	 * SQL DELETE statement for this table
	 */
	protected final String SQL_DELETE = "DELETE FROM " + getTableName() + " WHERE COD_UBICACION = ? AND COD_PRODUCTO = ?";

	/** 
	 * Index of column COD_UBICACION
	 */
	protected static final int COLUMN_UBI_ORIGEN = 1;

	/** 
	 * Index of column UBI_DESTINO
	 */
	protected static final int COLUMN_UBI_DESTINO = 2;
	
	/**
	 * Index of column COD_PRODUCTO
	 */
	protected static final int COLUMN_COD_PRODUCTO = 3;
	
	/** 
	 * Index of column CANTIDAD
	 */
	protected static final int COLUMN_CANTIDAD = 4;
	
	/** 
	 * Index of column FECHA
	 */
	protected static final int COLUMN_FECHA = 5;

	/** 
	 * Number of columns
	 */
	protected static final int NUMBER_OF_COLUMNS = 5;

	/** 
	 * Index of primary-key column COD_UBICACION and COD_PRODUCTO
	 */
	protected static final int PK_COLUMN_UBI_ORIGEN = 1;
	
	protected static final int PK_COLUMN_UBI_DESTINO = 2;
	
	protected static final int PK_COLUMN_COD_PRODUCTO = 3;
	
	protected static final int PK_COLUMN_FECHA = 4;

	/** 
	 * Inserts a new row in the MOVIMIENTOS table.
	 */
	public MovimientosPk insert(Movimientos dto) throws MovimientosDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			stmt = conn.prepareStatement( SQL_INSERT );
			if (dto.getUbiOrigen() != null) {
				stmt.setString( COLUMN_UBI_ORIGEN, dto.getUbiOrigen() );
			} else {
				stmt.setNull(COLUMN_UBI_ORIGEN, Types.INTEGER);
			}
			if (dto.getUbiDestino() != null) {
				stmt.setString( COLUMN_UBI_DESTINO, dto.getUbiDestino() );
			} else {
				stmt.setNull(COLUMN_UBI_DESTINO, Types.INTEGER);
			}
			
			if (dto.getCodProducto() != null) {
				stmt.setString( COLUMN_COD_PRODUCTO, dto.getCodProducto() );
			} else {
				stmt.setNull(COLUMN_COD_PRODUCTO, Types.INTEGER);
			}
		
			stmt.setString( COLUMN_CANTIDAD, String.valueOf(dto.getCantidad()));
			
			if (dto.getFecha() != null) {
				stmt.setString( COLUMN_FECHA, dto.getFecha() );
			} else {
				stmt.setNull(COLUMN_FECHA, Types.INTEGER);
			}
			System.out.println( "Executing " + SQL_INSERT + " with DTO: " + dto );
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
			return dto.createPk();
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new MovimientosDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new MovimientosDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Updates a single row in the MOVIMIENTOS table.
	 */
	public void update(MovimientosPk pk, Movimientos dto) throws MovimientosDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			System.out.println( "Executing " + SQL_UPDATE + " with DTO: " + dto );
			stmt = conn.prepareStatement( SQL_UPDATE );
			if (dto.getUbiOrigen() != null) {
				stmt.setString( COLUMN_UBI_ORIGEN, dto.getUbiOrigen() );
			} else {
				stmt.setNull(COLUMN_UBI_ORIGEN, Types.INTEGER);
			}
			if (dto.getUbiDestino() != null) {
				stmt.setString( COLUMN_UBI_DESTINO, dto.getUbiDestino() );
			} else {
				stmt.setNull(COLUMN_UBI_DESTINO, Types.INTEGER);
			}
			if (dto.getCodProducto() != null) {
				stmt.setString( COLUMN_COD_PRODUCTO, dto.getCodProducto() );
			} else {
				stmt.setNull(COLUMN_COD_PRODUCTO, Types.INTEGER);
			}
		
			stmt.setString( COLUMN_CANTIDAD, String.valueOf(dto.getCantidad()));
			
			if (dto.getFecha() != null) {
				stmt.setString( COLUMN_FECHA, dto.getFecha() );
			} else {
				stmt.setNull(COLUMN_FECHA, Types.INTEGER);
			}
			
			if (pk.getUbiOrigen() != null) {
				stmt.setString( 6, pk.getUbiOrigen() );
			} else {
				stmt.setNull(6, Types.INTEGER);
			}
			if (pk.getUbiDestino() != null) {
				stmt.setString( 7, pk.getUbiDestino() );
			} else {
				stmt.setNull(7, Types.INTEGER);
			}
			if (pk.getCodProducto() != null) {
				stmt.setString( 8, pk.getCodProducto() );
			} else {
				stmt.setNull(8, Types.INTEGER);
			}
			if (pk.getFecha() != null) {
				stmt.setString( 9, pk.getFecha() );
			} else {
				stmt.setNull(9, Types.INTEGER);
			}
		
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new MovimientosDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new MovimientosDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Deletes a single row in the MOVIMIENTOS table.
	 */
	public void delete(String pStrCodUbicacion, String pStrCodProducto) throws MovimientosDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			System.out.println( "Executing " + SQL_DELETE + " with PK: " + pStrCodUbicacion + ", " + pStrCodProducto );
			stmt = conn.prepareStatement( SQL_DELETE );
			if (pStrCodUbicacion != null) {
				stmt.setString( 1, pStrCodUbicacion );
			} else {
				stmt.setNull(1, Types.INTEGER);
			}
			if (pStrCodProducto != null) {
				stmt.setString( 2, pStrCodProducto );
			} else {
				stmt.setNull(2, Types.INTEGER);
			}
		
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new MovimientosDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new MovimientosDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Returns the rows from the MOVIMIENTOS table that matches the specified primary-key value.
	 */
	public Movimientos findByPrimaryKey(MovimientosPk pk) throws MovimientosDaoException
	{
		return findByPrimaryKey( pk );
	}

	/** 
	 * Returns all rows from the MOVIMIENTOS table.
	 */
	public List<Movimientos> findAll() throws MovimientosDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " ORDER BY UBI_ORIGEN", null );
	}

	/** 
	 * Returns all rows from the MOVIMIENTOS table that match the criteria 'CODIGO = :codigo'.
	 */
	public Movimientos findByPrimaryKey(Integer ubiOrigen, Integer ubiDestino, Integer codProducto, String fecha) throws MovimientosDaoException
	{
		List<Movimientos> ret = findByDynamicSelect( SQL_SELECT + " WHERE UBI_ORIGEN = ? AND UBI_DESTINO = ? AND COD_PRODUCTO = ?", new Object[] { ubiOrigen, ubiDestino, codProducto, fecha } );
		return ret.size()==0 ? null : ret.get(0);
	}

	/** 
	 * Returns all rows from the MOVIMIENTOS table that match the criteria 'CODIGO = :codigo'.
	 */
	public List<Movimientos> findWhereUbiOrigenEquals(String codUbicacion) throws MovimientosDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE UBI_ORIGEN = ? ORDER BY UBI_ORIGEN", new Object[] { codUbicacion } );
	}
	
	public List<Movimientos> findWhereUbiDestinoEquals(String codUbicacion) throws MovimientosDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE UBI_DESTINO = ? ORDER BY UBI_DESTINO", new Object[] { codUbicacion } );
	}
	
	public List<Movimientos> findWhereCodProductoEquals(String codProducto) throws MovimientosDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE COD_PRODUCTO = ? ORDER BY COD_PRODUCTO", new Object[] { codProducto } );
	}

	/** 
	 * Returns all rows from the MOVIMIENTOS table that match the criteria 'CANTIDAD = :cantidad'.
	 */
	public List<Movimientos> findWhereCantidadEquals(String cantidad) throws MovimientosDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE CANTIDAD = ? ORDER BY CANTIDAD", new Object[] { cantidad } );
	}

	/**
	 * Method 'MovimientosDaoImpl'
	 * 
	 */
	public MovimientosDaoImpl()
	{
	}

	/**
	 * Method 'MovimientosDaoImpl'
	 * 
	 * @param userConn
	 */
	public MovimientosDaoImpl(final java.sql.Connection userConn)
	{
		this.userConn = userConn;
	}

	/**
	 * Method 'getTableName'
	 * 
	 * @return String
	 */
	public String getTableName()
	{
		return "MOVIMIENTOS";
	}

	/** 
	 * Fetches a single row from the result set
	 */
	protected Movimientos fetchSingleResult(ResultSet rs) throws SQLException
	{
		if (rs.next()) {
			Movimientos dto = new Movimientos();
			populateDto( dto, rs);
			return dto;
		} else {
			return null;
		}
		
	}

	/** 
	 * Fetches multiple rows from the result set
	 */
	protected List<Movimientos> fetchMultiResults(ResultSet rs) throws SQLException
	{
		Collection resultList = new ArrayList();
		while (rs.next()) {
			Movimientos dto = new Movimientos();
			populateDto( dto, rs);
			resultList.add( dto );
		}
		return (List<Movimientos>) resultList;
	}

	/** 
	 * Populates a DTO with data from a ResultSet
	 */
	protected void populateDto(Movimientos dto, ResultSet rs) throws SQLException
	{
		dto.setUbiOrigen( rs.getString(COLUMN_UBI_ORIGEN));
		dto.setUbiDestino( rs.getString(COLUMN_UBI_DESTINO));
		dto.setCodProducto( rs.getString(COLUMN_COD_PRODUCTO));
		dto.setCantidad( rs.getString(COLUMN_CANTIDAD));
		dto.setFecha( rs.getString(COLUMN_FECHA));
	}

	/** 
	 * Returns all rows from the MOVIMIENTOS table that match the specified arbitrary SQL statement
	 */
	public List<Movimientos> findByDynamicSelect(String sql, Object[] sqlParams) throws MovimientosDaoException
	{
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			// construct the SQL statement
			final String SQL = sql;
		
		
			System.out.println( "Executing " + SQL );
			// prepare statement
			stmt = conn.prepareStatement( SQL );
		
			// bind parameters
			for (int i=0; sqlParams!=null && i<sqlParams.length; i++ ) {
				stmt.setObject( i+1, sqlParams[i] );
			}
		
		
			rs = stmt.executeQuery();
		
			// fetch the results
			return fetchMultiResults(rs);
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new MovimientosDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new MovimientosDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(rs);
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Returns all rows from the MOVIMIENTOS table that match the specified arbitrary SQL statement
	 */
	public List<Movimientos> findByDynamicWhere(String sql, Object[] sqlParams) throws MovimientosDaoException
	{
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			// construct the SQL statement
			final String SQL = SQL_SELECT + " WHERE " + sql;
		
		
			System.out.println( "Executing " + SQL );
			// prepare statement
			stmt = conn.prepareStatement( SQL );
		
			// bind parameters
			for (int i=0; sqlParams!=null && i<sqlParams.length; i++ ) {
				stmt.setObject( i+1, sqlParams[i] );
			}
		
		
			rs = stmt.executeQuery();
		
			// fetch the results
			return fetchMultiResults(rs);
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new MovimientosDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new MovimientosDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(rs);
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}
	
	public List<Movimientos> findByDynamicWhere(String pstrSQL)
			throws MovimientosDaoException {
		return findByDynamicWhere(pstrSQL, null);
	}

	public Movimientos findByPrimaryKey(String pStrUbiOrigen, String pStrUbiDestino, String pStrCodProducto, String pStrFecha) throws MovimientosDaoException {
		Movimientos mov = null;
		Connection con;
		try {
			con = ConexionBD.obtieneConexion();
			Statement s = con.createStatement();
			String strSQL = "SELECT * FROM MOVIMIENTOS WHERE UBI_ORIGEN = '" + pStrUbiOrigen + "' AND UBI_DESTINO = '" + pStrUbiDestino + "' AND COD_PRODUCTO = '" + pStrCodProducto + "' AND FECHA = '" + pStrFecha ;
			ResultSet rs = s.executeQuery(strSQL);
			if(rs.next()){
				System.out.println("ok");
				mov = new Movimientos();
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		return mov;
		//return findByPrimaryKey( codUbicacion, codProducto );
	}

	public List<Movimientos> findByDynamicSelect(String pstrSQL)
			throws MovimientosDaoException {
		return findByDynamicSelect(pstrSQL, null);
	}	
	
}
