/**
 * 
 */
package es.postigo.dao.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import es.postigo.dao.ProductoDao;
import es.postigo.delegate.InventarioDelegate;
import es.postigo.dto.Inventario;
import es.postigo.dto.Producto;
import es.postigo.dto.ProductoPk;
import es.postigo.exceptions.ProductoDaoException;
import es.postigo.jdbc.ResourceManager;

/**
 * @author informatica4
 *
 */
public class ProductoDaoImpl extends AbstractDataAccessObject implements ProductoDao
{
	/** 
	 * The factory class for this DAO has two versions of the create() method - one that
takes no arguments and one that takes a Connection argument. If the Connection version
is chosen then the connection will be stored in this attribute and will be used by all
calls to this DAO, otherwise a new Connection will be allocated for each operation.
	 */
	protected java.sql.Connection userConn;

	/** 
	 * All finder methods in this class use this SELECT constant to build their queries
	 */
	protected final String SQL_SELECT = "SELECT CODIGO, DESCRIPCION, LOTE, CANTIDAD, CODIGOEAN, CODIGO_COMPRAS, UNIDADES, STOCK_MIN FROM " + getTableName() + "";

	/** 
	 * SQL INSERT statement for this table
	 */
	protected final String SQL_INSERT = "INSERT INTO " + getTableName() + " ( CODIGO, DESCRIPCION, LOTE, CANTIDAD, CODIGOEAN, CODIGO_COMPRAS, UNIDADES, STOCK_MIN ) VALUES ( ?, ?, ?, ?, ?, ?, ?, ? )";

	/** 
	 * SQL UPDATE statement for this table
	 */
	protected final String SQL_UPDATE = "UPDATE " + getTableName() + " SET CODIGO = ?, DESCRIPCION = ?, LOTE = ?, CANTIDAD = ?, CODIGOEAN = ?, CODIGO_COMPRAS = ?, UNIDADES = ?, STOCK_MIN = ? WHERE CODIGO = ?";

	/** 
	 * SQL DELETE statement for this table
	 */
	protected final String SQL_DELETE = "DELETE FROM " + getTableName() + " WHERE CODIGO = ?";

	/** 
	 * Index of column CODIGO
	 */
	protected static final int COLUMN_CODIGO = 1;

	/** 
	 * Index of column DESCRIPCION
	 */
	protected static final int COLUMN_DESCRIPCION = 2;

	/** 
	 * Index of column LOTE
	 */
	protected static final int COLUMN_LOTE = 3;

	/** 
	 * Index of column CANTIDAD
	 */
	protected static final int COLUMN_CANTIDAD = 4;
	
	/** 
	 * Index of column CODIGOEAN
	 */
	protected static final int COLUMN_CODIGOEAN = 5;
	
	/** 
	 * Index of column CODIGO_COMPRAS
	 */
	protected static final int COLUMN_CODIGOCOMPRAS = 6;
	
	/** 
	 * Index of column CODIGO_UNIDADES
	 */
	protected static final int COLUMN_UNIDADES = 7;
	
	/** 
	 * Index of column COLUMN_STOCKMIN
	 */
	protected static final int COLUMN_STOCKMIN = 8;


	/** 
	 * Number of columns
	 */
	protected static final int NUMBER_OF_COLUMNS = 8;

	/** 
	 * Index of primary-key column CODIGO
	 */
	protected static final int PK_COLUMN_CODIGO = 1;

	/** 
	 * Inserts a new row in the PRODUCTO table.
	 */
	public ProductoPk insert(Producto dto) throws ProductoDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			stmt = conn.prepareStatement( SQL_INSERT );
			
			String lStrCodigo = String.valueOf(obtenerId("CODIGO", "PRODUCTO", "SUBSTR(codigo,1,3) = '000'", conn));
			if(lStrCodigo.length()<12){
				for(int i=lStrCodigo.length(); i<12; i++){
					lStrCodigo = "0" + lStrCodigo;
				}
			}
			dto.setCodigo(lStrCodigo);
			
			/**TODO: Mas tarde aadir una funcion para transformar dicho codigo secuencial a EAN13*/
			//int lIntChecksumEAN = generaEAN13(dto.getCodigo());
			int lIntChecksumEAN = generaChecksumEAN(dto.getCodigo());
			String lStrCodigoEAN = dto.getCodigo()+ lIntChecksumEAN;
			dto.setCodigoean(lStrCodigoEAN);
			
			if (dto.getCodigo() != null) {
				stmt.setString( COLUMN_CODIGO, dto.getCodigo() );
			} else {
				stmt.setNull(COLUMN_CODIGO, Types.INTEGER);
			}
		
			stmt.setString( COLUMN_DESCRIPCION, dto.getDescripcion() );
			stmt.setString( COLUMN_LOTE, dto.getLote() );
			stmt.setString( COLUMN_CANTIDAD, String.valueOf(dto.getCantidad()));
			stmt.setString( COLUMN_CODIGOEAN, dto.getCodigoean());
			stmt.setString( COLUMN_CODIGOCOMPRAS, dto.getCodigoCompras());
			stmt.setString( COLUMN_UNIDADES, dto.getUnidades());
			stmt.setString( COLUMN_STOCKMIN, String.valueOf(dto.getStockMin()));
			System.out.println( "Executing " + SQL_INSERT + " with DTO: " + dto );
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
			return dto.createPk();
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new ProductoDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new ProductoDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}
	
	/** 
	 * Inserts a new row in the PRODUCTO table.
	 * 
	 * Sobrecargamos el metodo para poder insertar automaticamente en el inventario cuando queremos ubicar en almacenes externos a POSTIGO
	 */
	public ProductoPk insert(Producto dto, String pStrUbicacion) throws ProductoDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			stmt = conn.prepareStatement( SQL_INSERT );
			
			String lStrCodigo = String.valueOf(obtenerId("CODIGO", "PRODUCTO", "SUBSTR(codigo,1,3) = '000'", conn));
			if(lStrCodigo.length()<12){
				for(int i=lStrCodigo.length(); i<12; i++){
					lStrCodigo = "0" + lStrCodigo;
				}
			}
			dto.setCodigo(lStrCodigo);
			
			/**TODO: Mas tarde aadir una funcion para transformar dicho codigo secuencial a EAN13*/
			//int lIntChecksumEAN = generaEAN13(dto.getCodigo());
			int lIntChecksumEAN = generaChecksumEAN(dto.getCodigo());
			String lStrCodigoEAN = dto.getCodigo()+ lIntChecksumEAN;
			dto.setCodigoean(lStrCodigoEAN);
			
			if (dto.getCodigo() != null) {
				stmt.setString( COLUMN_CODIGO, dto.getCodigo() );
			} else {
				stmt.setNull(COLUMN_CODIGO, Types.INTEGER);
			}
		
			stmt.setString( COLUMN_DESCRIPCION, dto.getDescripcion() );
			stmt.setString( COLUMN_LOTE, dto.getLote() );
			stmt.setString( COLUMN_CANTIDAD, String.valueOf(dto.getCantidad()));
			stmt.setString( COLUMN_CODIGOEAN, dto.getCodigoean());
			stmt.setString( COLUMN_CODIGOCOMPRAS, dto.getCodigoCompras());
			stmt.setString( COLUMN_UNIDADES, dto.getUnidades());
			stmt.setString( COLUMN_STOCKMIN, String.valueOf(dto.getStockMin()));
			System.out.println( "Executing " + SQL_INSERT + " with DTO: " + dto );
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
			
			/**Si nos han pasado la Ubicacin, debemos de insertar en la tabla inventario*/
			if(pStrUbicacion != null && pStrUbicacion != ""){
				Inventario lObjInventario = new Inventario();			        	
	        	lObjInventario.setCodUbicacion(pStrUbicacion); 
	        	lObjInventario.setCodProducto(dto.getCodigo());
	        	lObjInventario.setCodEanProducto(dto.getCodigoean());
	        	int lIntChecksumEAN2 = generaChecksumEAN(pStrUbicacion);
				String lStrCodigoUbicacionEAN = pStrUbicacion + lIntChecksumEAN2;
				lObjInventario.setCodEanUbicacion(lStrCodigoUbicacionEAN);
	        	lObjInventario.setCantidad(String.valueOf(dto.getCantidad()));
	        	lObjInventario.setCantidad(String.valueOf(dto.getStockMin()));
	        	InventarioDelegate lObjInventarioDelegate = new InventarioDelegate();	
	        	lObjInventarioDelegate.insert(lObjInventario);
    		}
			/***/
			
			return dto.createPk();			
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new ProductoDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new ProductoDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Updates a single row in the PRODUCTO table.
	 */
	public void update(ProductoPk pk, Producto dto) throws ProductoDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			System.out.println( "Executing " + SQL_UPDATE + " with DTO: " + dto );
			stmt = conn.prepareStatement( SQL_UPDATE );
			if (dto.getCodigo() != null) {
				stmt.setString( COLUMN_CODIGO, dto.getCodigo() );
			} else {
				stmt.setNull(COLUMN_CODIGO, Types.INTEGER);
			}
		
			stmt.setString( COLUMN_DESCRIPCION, dto.getDescripcion() );
			stmt.setString( COLUMN_LOTE, dto.getLote() );
			stmt.setString( COLUMN_CANTIDAD, String.valueOf(dto.getCantidad()));
			stmt.setString( COLUMN_CODIGOEAN, String.valueOf(dto.getCodigoean()));
			stmt.setString( COLUMN_CODIGOCOMPRAS, String.valueOf(dto.getCodigoCompras()));
			stmt.setString( COLUMN_UNIDADES, String.valueOf(dto.getUnidades()));
			stmt.setString( COLUMN_STOCKMIN, String.valueOf(dto.getStockMin()));
			if (pk.getCodigo() != null) {
				stmt.setString( 9, pk.getCodigo() );
			} else {
				stmt.setNull(9, Types.INTEGER);
			}
		
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new ProductoDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new ProductoDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Deletes a single row in the PRODUCTO table.
	 */
	public void delete(String pStrCodigo) throws ProductoDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			System.out.println( "Executing " + SQL_DELETE + " with PK: " + pStrCodigo );
			stmt = conn.prepareStatement( SQL_DELETE );
			if (pStrCodigo != null) {
				stmt.setString( 1, pStrCodigo );
			} else {
				stmt.setNull(1, Types.INTEGER);
			}
		
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new ProductoDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new ProductoDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Returns the rows from the PRODUCTO table that matches the specified primary-key value.
	 */
	public Producto findByPrimaryKey(ProductoPk pk) throws ProductoDaoException
	{
		return findByPrimaryKey( pk );
	}

	/** 
	 * Returns all rows from the PRODUCTO table.
	 */
	public List<Producto> findAll() throws ProductoDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " ORDER BY CODIGO", null );
	}

	/** 
	 * Returns all rows from the PRODUCTO table that match the criteria 'CODIGO = :codigo'.
	 */
	public Producto findByPrimaryKey(Long codigo) throws ProductoDaoException
	{
		List<Producto> ret = findByDynamicSelect( SQL_SELECT + " WHERE CODIGO = ?", new Object[] { codigo } );
		return ret.size()==0 ? null : ret.get(0);
	}

	/** 
	 * Returns all rows from the PRODUCTO table that match the criteria 'CODIGO = :codigo'.
	 */
	public List<Producto> findWhereCodigoEquals(String codigo) throws ProductoDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE CODIGO = ? ORDER BY CODIGO", new Object[] { codigo } );
	}

	/** 
	 * Returns all rows from the PRODUCTO table that match the criteria 'DESCRIPCION = :descripcion'.
	 */
	public List<Producto> findWhereDescripcionEquals(String descripcion) throws ProductoDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE DESCRIPCION = ? ORDER BY DESCRIPCION", new Object[] { descripcion } );
	}

	/** 
	 * Returns all rows from the PRODUCTO table that match the criteria 'LOTE = :lote'.
	 */
	public List<Producto> findWhereLoteEquals(String lote) throws ProductoDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE LOTE = ? ORDER BY LOTE", new Object[] { lote } );
	}

	/** 
	 * Returns all rows from the PRODUCTO table that match the criteria 'CANTIDAD = :cantidad'.
	 */
	public List<Producto> findWhereCantidadEquals(String cantidad) throws ProductoDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE CANTIDAD = ? ORDER BY CANTIDAD", new Object[] { cantidad } );
	}

	/**
	 * Method 'ProductoDaoImpl'
	 * 
	 */
	public ProductoDaoImpl()
	{
	}

	/**
	 * Method 'ProductoDaoImpl'
	 * 
	 * @param userConn
	 */
	public ProductoDaoImpl(final java.sql.Connection userConn)
	{
		this.userConn = userConn;
	}

	/**
	 * Method 'getTableName'
	 * 
	 * @return String
	 */
	public String getTableName()
	{
		return "PRODUCTO";
	}

	/** 
	 * Fetches a single row from the result set
	 */
	protected Producto fetchSingleResult(ResultSet rs) throws SQLException
	{
		if (rs.next()) {
			Producto dto = new Producto();
			populateDto( dto, rs);
			return dto;
		} else {
			return null;
		}
		
	}

	/** 
	 * Fetches multiple rows from the result set
	 */
	protected List<Producto> fetchMultiResults(ResultSet rs) throws SQLException
	{
		Collection resultList = new ArrayList();
		while (rs.next()) {
			Producto dto = new Producto();
			populateDto( dto, rs);
			resultList.add( dto );
		}
		return (List<Producto>) resultList;
	}

	/** 
	 * Populates a DTO with data from a ResultSet
	 */
	protected void populateDto(Producto dto, ResultSet rs) throws SQLException
	{
		dto.setCodigo( rs.getString(COLUMN_CODIGO));
		dto.setDescripcion( rs.getString(COLUMN_DESCRIPCION));
		dto.setLote( rs.getString(COLUMN_LOTE));
		dto.setCantidad( Integer.parseInt(rs.getString(COLUMN_CANTIDAD)));
		dto.setCodigoean( rs.getString(COLUMN_CODIGOEAN));
		dto.setCodigoCompras( rs.getString(COLUMN_CODIGOCOMPRAS));
		dto.setUnidades( rs.getString(COLUMN_UNIDADES));
		dto.setStockMin(Integer.parseInt(rs.getString(COLUMN_STOCKMIN)));
	}

	/** 
	 * Returns all rows from the PRODUCTO table that match the specified arbitrary SQL statement
	 */
	public List<Producto> findByDynamicSelect(String sql, Object[] sqlParams) throws ProductoDaoException
	{
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			// construct the SQL statement
			final String SQL = sql;
		
		
			System.out.println( "Executing " + SQL );
			// prepare statement
			stmt = conn.prepareStatement( SQL );
		
			// bind parameters
			for (int i=0; sqlParams!=null && i<sqlParams.length; i++ ) {
				stmt.setObject( i+1, sqlParams[i] );
			}
		
		
			rs = stmt.executeQuery();
		
			// fetch the results
			return fetchMultiResults(rs);
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new ProductoDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new ProductoDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(rs);
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Returns all rows from the PRODUCTO table that match the specified arbitrary SQL statement
	 */
	public List<Producto> findByDynamicWhere(String sql, Object[] sqlParams) throws ProductoDaoException
	{
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			// construct the SQL statement
			final String SQL = SQL_SELECT + " WHERE " + sql;
		
		
			System.out.println( "Executing " + SQL );
			// prepare statement
			stmt = conn.prepareStatement( SQL );
		
			// bind parameters
			for (int i=0; sqlParams!=null && i<sqlParams.length; i++ ) {
				stmt.setObject( i+1, sqlParams[i] );
			}
		
		
			rs = stmt.executeQuery();
		
			// fetch the results
			return fetchMultiResults(rs);
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new ProductoDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new ProductoDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(rs);
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}
	
	public List<Producto> findByDynamicWhere(String pstrSQL)
			throws ProductoDaoException {
		return findByDynamicWhere(pstrSQL, null);
	}

	public Producto findByPrimaryKey(String codigo) throws ProductoDaoException {
		/**TODO: Probar con Long o Double xa ver si estos codigos tan largos sobrepasan el tamao maximo de un Integer*/
		return findByPrimaryKey( Long.parseLong(codigo));
	}

	public List<Producto> findByDynamicSelect(String pstrSQL)
			throws ProductoDaoException {
		return findByDynamicSelect(pstrSQL, null);
	}
	
	public long obtenerId(String pStrIdCampo, String pStrTabla, String pStrWhere, Connection con) throws ProductoDaoException {

		String lStrSQL = " SELECT MAX(" + pStrIdCampo + ") FROM " + pStrTabla;
		if(pStrWhere.length()>0){
			lStrSQL += " WHERE "+ pStrWhere;
		}
		//return (getLong(lStrSQL) + 1);
		PreparedStatement stmt = null;
		ResultSet rs = null;
		long max = 0;
		try {
			stmt = con.prepareStatement(lStrSQL);
			rs = stmt.executeQuery();
			while (rs.next()) {
				max = Long.parseLong(rs.getString("MAX(CODIGO)"))+1;	
			}
		} catch (SQLException e) {
			e.printStackTrace();
			throw new ProductoDaoException( "SQLException: " + e.getMessage(), e );
		}
		return max;		
	}
	
	/**TODO: String index out of range: -10*/
	public int generaEAN13(String pStrCodigo) throws ProductoDaoException {
		// Clculo del dgito de control EAN
		int iSum = 0;
		int iSumInpar = 0;
		int iDigit = 0;
		String EAN = pStrCodigo;
		EAN = "000000000001";
				 
 		for (int i = EAN.length(); i > 0; i--)
		{
		 iDigit = Integer.parseInt(EAN.substring(i - 1, i));
		 if (i % 2 != 0)
		 {
		    iSumInpar += iDigit;
		 }
		 else
		 {
		    iSum += iDigit;
		 }
		}
		 
		iDigit = (iSumInpar*3) + iSum ;
		 
		int iCheckSum = (10 - (iDigit % 10)) % 10;
		System.out.println("Digito de control: " + iCheckSum);
		return iCheckSum;
	}
	
	/**
	 * Clculo del dgito de control EAN
	 * @param lStrCodigo
	 * @return checksum
	 */
	public int generaChecksumEAN(String lStrCodigo){
		int lIntPares = 0;
		int lIntImpares = 0;
		int lIntSuma = 0;
		int lIntResultado = 0;
		
		//lStrCodigo = reverse(lStrCodigo);
		
		for (int i = lStrCodigo.length(); i > 0; i--) {
			int lIntDigito = Integer.parseInt(lStrCodigo.substring(i - 1, i));
			if (i % 2 != 0) {
				lIntPares += lIntDigito;
			} else {
				lIntImpares += lIntDigito;
			}
		}
		
		lIntResultado = (10 - (((lIntImpares * 3) + lIntPares) % 10)) % 10;
		
		return lIntResultado;
	}
	
	public String reverse(String str) {
        if ((null == str) || (str.length() <= 1)) {
            return str;
        }
        StringBuffer reverse = new StringBuffer(str.length());
        for (int i = str.length() - 1; i >= 0; i--) {
          reverse.append(str.charAt(i));
        }
        return reverse.toString();
    }

	public Producto findByCodigoEAN(String codigoEAN)
			throws ProductoDaoException {
		List<Producto> ret = findByDynamicSelect( SQL_SELECT + " WHERE CODIGOEAN = ?", new Object[] { codigoEAN } );
		return ret.size()==0 ? null : ret.get(0);		
	}

	public String obtenerUltimoElementoInsertado() throws ProductoDaoException {
		String lStrResult = "";
		String lStrSQL = " SELECT MAX(CODIGO) FROM " + getTableName();
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		try {
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
			stmt = conn.prepareStatement(lStrSQL);
			rs = stmt.executeQuery();
			while (rs.next()) {
				lStrResult = rs.getString("MAX(CODIGO)");	
			}
		} catch (SQLException e) {
			e.printStackTrace();
			throw new ProductoDaoException( "SQLException: " + e.getMessage(), e );
		}
		return lStrResult;
	}
		
}
