/**
 * 
 */
package es.postigo.dao.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import es.postigo.dao.UbicacionDao;
import es.postigo.delegate.InventarioDelegate;
import es.postigo.dto.Inventario;
import es.postigo.dto.Ubicacion;
import es.postigo.dto.UbicacionPk;
import es.postigo.exceptions.ProductoDaoException;
import es.postigo.exceptions.UbicacionDaoException;
import es.postigo.jdbc.ResourceManager;

/**
 * @author informatica4
 *
 */
public class UbicacionDaoImpl extends AbstractDataAccessObject implements UbicacionDao
{
	/** 
	 * The factory class for this DAO has two versions of the create() method - one that
takes no arguments and one that takes a Connection argument. If the Connection version
is chosen then the connection will be stored in this attribute and will be used by all
calls to this DAO, otherwise a new Connection will be allocated for each operation.
	 */
	protected java.sql.Connection userConn;

	/** 
	 * All finder methods in this class use this SELECT constant to build their queries
	 */
	protected final String SQL_SELECT = "SELECT CODIGO, CODIGOEAN, DESCRIPCION FROM " + getTableName() + "";

	/** 
	 * SQL INSERT statement for this table
	 */
	protected final String SQL_INSERT = "INSERT INTO " + getTableName() + " ( CODIGO, CODIGOEAN, DESCRIPCION ) VALUES ( ?, ?, ? )";

	/** 
	 * SQL UPDATE statement for this table
	 */
	protected final String SQL_UPDATE = "UPDATE " + getTableName() + " SET CODIGO = ?, CODIGOEAN = ?, DESCRIPCION = ? WHERE CODIGO = ?";

	/** 
	 * SQL DELETE statement for this table
	 */
	protected final String SQL_DELETE = "DELETE FROM " + getTableName() + " WHERE CODIGO = ?";

	/** 
	 * Index of column CODIGO
	 */
	protected static final int COLUMN_CODIGO = 1;

	/** 
	 * Index of column CODIGOEAN
	 */
	protected static final int COLUMN_CODIGOEAN = 2;
	
	/** 
	 * Index of column DESCRIPCION
	 */
	protected static final int COLUMN_DESCRIPCION = 3;

	/** 
	 * Number of columns
	 */
	protected static final int NUMBER_OF_COLUMNS = 3;

	/** 
	 * Index of primary-key column CODIGO
	 */
	protected static final int PK_COLUMN_CODIGO = 1;

	
	/** 
	 * Inserts a new row in the UBICACION table.
	 */
	public UbicacionPk insert(Ubicacion dto) throws UbicacionDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			stmt = conn.prepareStatement( SQL_INSERT );
			
			String lStrCodigo = String.valueOf(obtenerIdUbicacion("CODIGO", "UBICACION", "SUBSTR(codigo,1,3) != '000' AND SUBSTR(codigo,1,3) != '999' AND SUBSTR(codigo,1,3) != '998'", conn));
			if(lStrCodigo.length()<12){
				for(int i=lStrCodigo.length(); i<12; i++){
					lStrCodigo = "0" + lStrCodigo;
				}
			}
			dto.setCodigo(lStrCodigo);
			
			/**TODO: Mas tarde aadir una funcion para transformar dicho codigo secuencial a EAN13*/
			//int lIntChecksumEAN = generaEAN13(dto.getCodigo());
			int lIntChecksumEAN = generaChecksumEAN(dto.getCodigo());
			String lStrCodigoEAN = dto.getCodigo()+ lIntChecksumEAN;
			dto.setCodigoean(lStrCodigoEAN);
			
			if (dto.getCodigo() != null) {
				stmt.setString( COLUMN_CODIGO, dto.getCodigo() );
			} else {
				stmt.setNull(COLUMN_CODIGO, Types.INTEGER);
			}
		
			stmt.setString( COLUMN_DESCRIPCION, dto.getDescripcion() );
			stmt.setString( COLUMN_CODIGOEAN, dto.getCodigoean());
			System.out.println( "Executing " + SQL_INSERT + " with DTO: " + dto );
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
			return dto.createPk();
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new UbicacionDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new UbicacionDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Updates a single row in the UBICACION table.
	 */
	public void update(UbicacionPk pk, Ubicacion dto) throws UbicacionDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			System.out.println( "Executing " + SQL_UPDATE + " with DTO: " + dto );
			stmt = conn.prepareStatement( SQL_UPDATE );
			if (dto.getCodigo() != null) {
				stmt.setString( COLUMN_CODIGO, dto.getCodigo() );
			} else {
				stmt.setNull(COLUMN_CODIGO, Types.INTEGER);
			}
		
			stmt.setString( COLUMN_DESCRIPCION, dto.getDescripcion() );
			stmt.setString( COLUMN_CODIGOEAN, String.valueOf(dto.getCodigoean()));
			if (pk.getCodigo() != null) {
				stmt.setString( 4, pk.getCodigo() );
			} else {
				stmt.setNull(4, Types.INTEGER);
			}
		
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new UbicacionDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new UbicacionDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Deletes a single row in the UBICACION table.
	 */
	public void delete(String pStrCodigo) throws UbicacionDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			System.out.println( "Executing " + SQL_DELETE + " with PK: " + pStrCodigo );
			stmt = conn.prepareStatement( SQL_DELETE );
			if (pStrCodigo != null) {
				stmt.setString( 1, pStrCodigo );
			} else {
				stmt.setNull(1, Types.INTEGER);
			}
		
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new UbicacionDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new UbicacionDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Returns the rows from the UBICACION table that matches the specified primary-key value.
	 */
	public Ubicacion findByPrimaryKey(UbicacionPk pk) throws UbicacionDaoException
	{
		return findByPrimaryKey( pk );
	}

	/** 
	 * Returns all rows from the UBICACION table.
	 */
	public List<Ubicacion> findAll() throws UbicacionDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " ORDER BY CODIGO", null );
	}

	/** 
	 * Returns all rows from the UBICACION table that match the criteria 'CODIGO = :codigo'.
	 */
	public Ubicacion findByPrimaryKey(Long codigo) throws UbicacionDaoException
	{
		List<Ubicacion> ret = findByDynamicSelect( SQL_SELECT + " WHERE CODIGO = ?", new Object[] { codigo } );
		return ret.size()==0 ? null : ret.get(0);
	}

	/** 
	 * Returns all rows from the UBICACION table that match the criteria 'CODIGO = :codigo'.
	 */
	public List<Ubicacion> findWhereCodigoEquals(String codigo) throws UbicacionDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE CODIGO = ? ORDER BY CODIGO", new Object[] { codigo } );
	}

	/** 
	 * Returns all rows from the UBICACION table that match the criteria 'DESCRIPCION = :descripcion'.
	 */
	public List<Ubicacion> findWhereDescripcionEquals(String descripcion) throws UbicacionDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " WHERE DESCRIPCION = ? ORDER BY DESCRIPCION", new Object[] { descripcion } );
	}


	/**
	 * Method 'UbicacionDaoImpl'
	 * 
	 */
	public UbicacionDaoImpl()
	{
	}

	/**
	 * Method 'UbicacionDaoImpl'
	 * 
	 * @param userConn
	 */
	public UbicacionDaoImpl(final java.sql.Connection userConn)
	{
		this.userConn = userConn;
	}

	/**
	 * Method 'getTableName'
	 * 
	 * @return String
	 */
	public String getTableName()
	{
		return "UBICACION";
	}

	/** 
	 * Fetches a single row from the result set
	 */
	protected Ubicacion fetchSingleResult(ResultSet rs) throws SQLException
	{
		if (rs.next()) {
			Ubicacion dto = new Ubicacion();
			populateDto( dto, rs);
			return dto;
		} else {
			return null;
		}
		
	}

	/** 
	 * Fetches multiple rows from the result set
	 */
	protected List<Ubicacion> fetchMultiResults(ResultSet rs) throws SQLException
	{
		Collection resultList = new ArrayList();
		while (rs.next()) {
			Ubicacion dto = new Ubicacion();
			populateDto( dto, rs);
			resultList.add( dto );
		}
		return (List<Ubicacion>) resultList;
	}

	/** 
	 * Populates a DTO with data from a ResultSet
	 */
	protected void populateDto(Ubicacion dto, ResultSet rs) throws SQLException
	{
		dto.setCodigo( rs.getString(COLUMN_CODIGO));
		dto.setDescripcion( rs.getString(COLUMN_DESCRIPCION));
		dto.setCodigoean( rs.getString(COLUMN_CODIGOEAN));		
	}

	/** 
	 * Returns all rows from the UBICACION table that match the specified arbitrary SQL statement
	 */
	public List<Ubicacion> findByDynamicSelect(String sql, Object[] sqlParams) throws UbicacionDaoException
	{
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			// construct the SQL statement
			final String SQL = sql;
		
		
			System.out.println( "Executing " + SQL );
			// prepare statement
			stmt = conn.prepareStatement( SQL );
		
			// bind parameters
			for (int i=0; sqlParams!=null && i<sqlParams.length; i++ ) {
				stmt.setObject( i+1, sqlParams[i] );
			}
		
		
			rs = stmt.executeQuery();
		
			// fetch the results
			return fetchMultiResults(rs);
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new UbicacionDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new UbicacionDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(rs);
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Returns all rows from the UBICACION table that match the specified arbitrary SQL statement
	 */
	public List<Ubicacion> findByDynamicWhere(String sql, Object[] sqlParams) throws UbicacionDaoException
	{
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			// construct the SQL statement
			final String SQL = SQL_SELECT + " WHERE " + sql;
		
		
			System.out.println( "Executing " + SQL );
			// prepare statement
			stmt = conn.prepareStatement( SQL );
		
			// bind parameters
			for (int i=0; sqlParams!=null && i<sqlParams.length; i++ ) {
				stmt.setObject( i+1, sqlParams[i] );
			}
		
		
			rs = stmt.executeQuery();
		
			// fetch the results
			return fetchMultiResults(rs);
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new UbicacionDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new UbicacionDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(rs);
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}
	
	public List<Ubicacion> findByDynamicWhere(String pstrSQL)
			throws UbicacionDaoException {
		return findByDynamicWhere(pstrSQL, null);
	}

	public Ubicacion findByPrimaryKey(String codigo) throws UbicacionDaoException {
		/**TODO: Probar con Long o Double xa ver si estos codigos tan largos sobrepasan el tamao maximo de un Integer*/
		return findByPrimaryKey( Long.parseLong(codigo));
	}

	public List<Ubicacion> findByDynamicSelect(String pstrSQL)
			throws UbicacionDaoException {
		return findByDynamicSelect(pstrSQL, null);
	}
	
	public long obtenerId(String pStrIdCampo, String pStrTabla, String pStrWhere, Connection con) throws UbicacionDaoException {

		String lStrSQL = " SELECT MAX(" + pStrIdCampo + ") FROM " + pStrTabla;
		if(pStrWhere.length()>0){
			lStrSQL += " WHERE "+ pStrWhere;
		}
		//return (getLong(lStrSQL) + 1);
		PreparedStatement stmt = null;
		ResultSet rs = null;
		long max = 0;
		try {
			stmt = con.prepareStatement(lStrSQL);
			rs = stmt.executeQuery();
			while (rs.next()) {
				max = Long.parseLong(rs.getString("MAX(CODIGO)"))+1;	
			}
		} catch (SQLException e) {
			e.printStackTrace();
			throw new UbicacionDaoException( "SQLException: " + e.getMessage(), e );
		}
		return max;		
	}
	
	public long obtenerIdUbicacion(String pStrIdCampo, String pStrTabla, String pStrWhere, Connection con) throws UbicacionDaoException {

		String lStrSQL = " SELECT MAX(" + pStrIdCampo + ") FROM " + pStrTabla;
		if(pStrWhere.length()>0){
			lStrSQL += " WHERE "+ pStrWhere;
		}
		//return (getLong(lStrSQL) + 1);
		PreparedStatement stmt = null;
		ResultSet rs = null;
		long max = 0;
		try {
			stmt = con.prepareStatement(lStrSQL);
			rs = stmt.executeQuery();
			while (rs.next()) {
				max = Long.parseLong(rs.getString("MAX(CODIGO)"))+1000000000;	
			}
		} catch (SQLException e) {
			e.printStackTrace();
			throw new UbicacionDaoException( "SQLException: " + e.getMessage(), e );
		}
		return max;		
	}

	
	/**TODO: String index out of range: -10*/
	public int generaEAN13(String pStrCodigo) throws UbicacionDaoException {
		// Clculo del dgito de control EAN
		int iSum = 0;
		int iSumInpar = 0;
		int iDigit = 0;
		String EAN = pStrCodigo;
		EAN = "000000000001";
				 
 		for (int i = EAN.length(); i > 0; i--)
		{
		 iDigit = Integer.parseInt(EAN.substring(i - 1, i));
		 if (i % 2 != 0)
		 {
		    iSumInpar += iDigit;
		 }
		 else
		 {
		    iSum += iDigit;
		 }
		}
		 
		iDigit = (iSumInpar*3) + iSum ;
		 
		int iCheckSum = (10 - (iDigit % 10)) % 10;
		System.out.println("Digito de control: " + iCheckSum);
		return iCheckSum;
	}
	
	/**
	 * Clculo del dgito de control EAN
	 * @param lStrCodigo
	 * @return checksum
	 */
	public int generaChecksumEAN(String lStrCodigo){
		int lIntPares = 0;
		int lIntImpares = 0;
		int lIntSuma = 0;
		int lIntResultado = 0;
		
		//lStrCodigo = reverse(lStrCodigo);
		
		for (int i = lStrCodigo.length(); i > 0; i--) {
			int lIntDigito = Integer.parseInt(lStrCodigo.substring(i - 1, i));
			if (i % 2 != 0) {
				lIntPares += lIntDigito;
			} else {
				lIntImpares += lIntDigito;
			}
		}
		
		lIntResultado = (10 - (((lIntImpares * 3) + lIntPares) % 10)) % 10;
		
		return lIntResultado;
	}
	
	public String reverse(String str) {
        if ((null == str) || (str.length() <= 1)) {
            return str;
        }
        StringBuffer reverse = new StringBuffer(str.length());
        for (int i = str.length() - 1; i >= 0; i--) {
          reverse.append(str.charAt(i));
        }
        return reverse.toString();
    }

	public Ubicacion findByCodigoEAN(String codigoEAN)
			throws UbicacionDaoException {
		List<Ubicacion> ret = findByDynamicSelect( SQL_SELECT + " WHERE CODIGOEAN = ?", new Object[] { codigoEAN } );
		return ret.size()==0 ? null : ret.get(0);		
	}

	public String obtenerUltimoElementoInsertado() throws UbicacionDaoException {
		String lStrResult = "";
		String lStrSQL = " SELECT MAX(CODIGO) FROM " + getTableName() + " WHERE SUBSTR(codigo,1,3) != '999' AND SUBSTR(codigo,1,3) != '998'";
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		try {
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
			stmt = conn.prepareStatement(lStrSQL);
			rs = stmt.executeQuery();
			while (rs.next()) {
				lStrResult = rs.getString("MAX(CODIGO)");	
			}
		} catch (SQLException e) {
			e.printStackTrace();
			throw new UbicacionDaoException( "SQLException: " + e.getMessage(), e );
		}
		return lStrResult;
	}
	
}
