/**
 * 
 */
package es.postigo.delegate;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.List;

import es.postigo.dto.Inventario;
import es.postigo.dtov.InventarioDTOV;
import es.postigo.exceptions.ApplicationException;
import es.postigo.exceptions.InventarioDaoException;
import es.postigo.factoria.InventarioDaoFactory;
import es.postigo.utility.ConexionBD;

/**
 * @author informatica4
 *
 */
public class InventarioDelegate {

	public InventarioDelegate() throws Exception {

	}

	public List<Inventario> obtenerListado(String pStrSql) throws ApplicationException {

		List<Inventario> listaProductos = new ArrayList<Inventario>();
		Connection con;
		
		try {
			//Class.forName("com.ibm.as400.access.AS400JDBCDriver");
			//Connection con = DriverManager.getConnection("jdbc:as400://192.168.0.3","ipcs","ipcs");
			con = ConexionBD.obtieneConexion();
			System.out.println("Conectado correctamente");
			
			PreparedStatement statement = con.prepareStatement(pStrSql);
			ResultSet rs = statement.executeQuery();
			
			while (rs.next()) {
				Inventario pu = new Inventario();
				
				pu.setCodUbicacion(rs.getString("COD_UBICACION"));
				pu.setDescCodUbicacion(rs.getString("DESC_COD_UBICACION"));
				pu.setCodProducto(rs.getString("COD_PRODUCTO"));
				pu.setDescCodProducto(rs.getString("DESC_COD_PRODUCTO"));
				pu.setCantidad(rs.getString("CANTIDAD"));
				
				listaProductos.add(pu);				
			}
			
			rs.close();
			con.close();
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return listaProductos;

	}

	public InventarioDTOV getRegistroV(String pStrCodUbicacion, String pStrCodProducto) throws ApplicationException, InventarioDaoException {

		return (InventarioDTOV) InventarioDaoFactory.create().findByPrimaryKey(pStrCodUbicacion, pStrCodProducto);

	}

	public void insert(Inventario pObjInventario) throws ApplicationException, InventarioDaoException {

		InventarioDaoFactory.create().insert(pObjInventario);

	}

	public void update(Inventario pObjInventario) throws ApplicationException, InventarioDaoException {

		InventarioDaoFactory.create().update(pObjInventario.createPk(),pObjInventario);

	}
	
	public void incrementarInventario(Inventario pObjInventario) throws ApplicationException, InventarioDaoException {

		InventarioDaoFactory.create().incrementarInventario(pObjInventario.createPk(),pObjInventario);

	}
	
	public void incrementarInventario(Inventario pObjInventario, String pStrCodUbicacionOrigen) throws ApplicationException, InventarioDaoException {

		InventarioDaoFactory.create().incrementarInventario(pObjInventario.createPk(), pObjInventario, pStrCodUbicacionOrigen);

	}
	
	public void decrementarInventario(Inventario pObjInventario) throws ApplicationException, InventarioDaoException {

		InventarioDaoFactory.create().decrementarInventario(pObjInventario.createPk(),pObjInventario);

	}
	
	public void decrementarInventario(Inventario pObjInventario, String pStrCodUbicacionDestino) throws ApplicationException, InventarioDaoException {

		InventarioDaoFactory.create().decrementarInventario(pObjInventario.createPk(), pObjInventario, pStrCodUbicacionDestino);

	}

	public Inventario getRegistro(String pStrCodUbicacion, String pStrCodProducto) throws ApplicationException, InventarioDaoException {

		return InventarioDaoFactory.create().findByPrimaryKey(pStrCodUbicacion, pStrCodProducto);

	}
	
	public Inventario getRegistroPorCodigosEAN(String pStrCodEanUbicacion, String pStrCodEanProducto) throws ApplicationException, InventarioDaoException {

		return InventarioDaoFactory.create().findByCodigosEAN(pStrCodEanUbicacion, pStrCodEanProducto);

	}

	public void delete(String pStrCodUbicacion, String pStrCodProducto) throws ApplicationException, InventarioDaoException {

		InventarioDaoFactory.create().delete(pStrCodUbicacion, pStrCodProducto);

	}
}
