/**
 * 
 */
package es.postigo.delegate;

import java.util.ArrayList;
import java.util.HashMap;

import es.postigo.dto.MetadatoDTO;
import es.postigo.exceptions.ApplicationCoreException;
import es.postigo.service.TCService;
import es.postigo.servicelocator.ServiceLocatorCore;

/**
 * @author informatica4
 *
 */
public class TCDelegate {

	/**
	 * Obtener numero registros.
	 * 
	 * @param pStrTabla
	 *            el string Tabla
	 * 
	 * @return un objeto int
	 * 
	 * @throws ApplicationCoreException
	 *             the application exception
	 */
	public int obtenerNumeroRegistros(String pStrTabla) throws ApplicationCoreException {

		TCService lObjService = ServiceLocatorCore.getTcService();
		return lObjService.obtenerNumeroRegistros(pStrTabla);
	}

	/**
	 * Obtener numero registros actual.
	 * 
	 * @param pStrTabla
	 *            el string Tabla
	 * @param pStrCondicion
	 *            el string Condicion
	 * 
	 * @return un objeto int
	 * 
	 * @throws ApplicationCoreException
	 *             the application exception
	 */
	public int obtenerNumeroRegistrosActual(String pStrTabla, String pStrCondicion) throws ApplicationCoreException {

		TCService lObjService = ServiceLocatorCore.getTcService();
		return lObjService.obtenerNumeroRegistrosActual(pStrTabla, pStrCondicion);
	}

	/**
	 * Obtiene datos lupa.
	 * 
	 * @param pStrSQL
	 *            el string SQL
	 * @param pObjColumnas
	 *            el objeto Columnas
	 * 
	 * @return un objeto HashMap
	 * 
	 * @throws ApplicationCoreException
	 *             the application exception
	 */
//	public HashMap obtieneDatosLupa(String pStrSQL, MetadatoDTO[] pObjColumnas) throws ApplicationCoreException {
//
//		LupaService lObjService = ServiceLocatorCore.getLupaService();
//		return lObjService.obtieneDatosLupa(pStrSQL, pObjColumnas);
//	}

	/**
	 * Obtiene datos.
	 * 
	 * @param pStrTabla
	 *            el string Tabla
	 * @param pStrColumnas
	 *            el string Columnas
	 * @param pStrWhere
	 *            el string Where
	 * @param pObjColumnas
	 *            el objeto Columnas
	 * 
	 * @return un objeto HashMap
	 * 
	 * @throws ApplicationCoreException
	 *             the application exception
	 */
	public HashMap obtieneDatos(String pStrTabla, String pStrColumnas, String pStrWhere, MetadatoDTO[] pObjColumnas)
			throws ApplicationCoreException {

		TCService lObjService = ServiceLocatorCore.getTcService();
		return lObjService.obtieneDatos(pStrTabla, pStrColumnas, pStrWhere, pObjColumnas);
	}

	/**
	 * Obtener hash map.
	 * 
	 * @param pStrElegidas
	 *            el string Elegidas
	 * @param pStrTabla
	 *            el string Tabla
	 * @param pStrWhere
	 *            el string Where
	 * @param pStrCondicion
	 *            el string Condicion
	 * @param pIntCursor
	 *            el integer Cursor
	 * @param pIntNumRegistros
	 *            el integer NumRegistros
	 * 
	 * @return un objeto HashMap
	 * 
	 * @throws ApplicationCoreException
	 *             the application exception
	 */
	public HashMap obtenerHashMap(String pStrElegidas, String pStrTabla, String pStrWhere, String pStrCondicion,
			int pIntCursor, int pIntNumRegistros) throws ApplicationCoreException {

		TCService lObjService = ServiceLocatorCore.getTcService();
		return lObjService.obtenerHashMap(pStrElegidas, pStrTabla, pStrWhere, pStrCondicion, pIntCursor,
				pIntNumRegistros);
	}
	
	/**
	 * Obtener intercambio
	 * 
	 * @param pStrTabla
	 * 			el string Tabla
	 * 
	 * @param pStrElegidas
	 * 			el string Elegidas
	 * 
	 * @param pStrPrefijo
	 * 			Prefijo opcional para buscar las etiquetas en el properties
	 * 
	 * @return un objeto MetadatoDTO[]
	 * 
	 * @throws ApplicationCoreException
	 * 				the application exception
	 */
	public MetadatoDTO[] obtenerIntercambio(String pStrTabla, String pStrElegidas, String pStrPrefijo) throws ApplicationCoreException {

		TCService lObjService = ServiceLocatorCore.getTcService();
		return lObjService.obtenerIntercambio(pStrTabla, pStrElegidas, pStrPrefijo);
	}	

	/**
	 * Obtiene el todas columnas.
	 * 
	 * @param pStrTabla
	 *            el string Tabla
	 * 
	 * @return el todas columnas
	 * 
	 * @throws ApplicationCoreException
	 *             the application exception
	 */
	public HashMap getTodasColumnas(String pStrTabla) throws ApplicationCoreException {

		TCService lObjService = ServiceLocatorCore.getTcService();
		return lObjService.obtenerHashOrden(pStrTabla);
	}

	/**
	 * Obtener visibles no visibles.
	 * 
	 * @param pStrTabla
	 *            el string Tabla
	 * @param pStrVisibles
	 *            el string Visibles
	 * 
	 * @return un objeto ArrayList
	 * 
	 * @throws ApplicationCoreException
	 *             the application exception
	 */
	public ArrayList obtenerVisiblesNoVisibles(String pStrTabla, String pStrVisibles) throws ApplicationCoreException {

		TCService lObjService = ServiceLocatorCore.getTcService();
		return lObjService.obtenerVisiblesNoVisibles(pStrTabla, pStrVisibles);
	}

	/**
	 * Obtiene el pks.
	 * 
	 * @param pStrSqlTC
	 *            el string SqlTC
	 * @param pObjCampos
	 *            el objeto Campos
	 * @param pStrNumrow
	 *            el string Numrow
	 * 
	 * @return el pks
	 * 
	 * @throws Throwable
	 *             the throwable
	 */
//	public HashMap getPks(String pStrSqlTC, HashMap pObjCampos, String pStrNumrow) throws Throwable {
//
//		AvanceRegistroService lObjService = ServiceLocatorCore.getAvanceRegistroService();
//		return lObjService.getPks(pStrSqlTC, pObjCampos, pStrNumrow);
//	}

	/**
	 * Obtiene descripcion.
	 * 
	 * @param pStrSQL
	 *            el string SQL
	 * @param pIntNumColumnas
	 *            el integer NumColumnas
	 * 
	 * @return un objeto String
	 * 
	 * @throws Throwable
	 *             the throwable
	 */
	public String obtieneDescripcion(String pStrSQL, int pIntNumColumnas) throws Throwable {
		return ServiceLocatorCore.getTcService().obtieneDescripcion(pStrSQL, pIntNumColumnas);
	}

	/***************************************************************************
	 * Obtiene un xml a partir de una lista de datos dependientes de otra tabla
	 * 
	 * @param pStrSQL
	 *            SQL que se va a ejecutar
	 * @return
	 * @throws Throwable
	 */
	public String obtieneComboDependiente(String pStrSQL) throws Throwable {
		return ServiceLocatorCore.getTcService().obtieneComboDependiente(pStrSQL);
	}
}
