package es.postigo.dto;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;

/****************************************************************************
 * Clase que lee propiedades de un archivo .properties
 * El constructor carga las propiedades. Si hay error, queda almacenado en mensajeError;
 * si no lo hubiere, mensajeError permanece siendo null.
 ****************************************************************************/
public class Propiedades {

   private String ficheroParametros = "parametros.properties";
   private Properties prop = new Properties();
   public String mensajeError;

   /***************************************************************************
    * Constructor que carga en atributo 'prop' el archivo de propiedades
    ***************************************************************************/
   public Propiedades( String pathContexto ) {
      try {
	 URL url = new URL( "file:" + pathContexto + ficheroParametros ); // Abro URL
	 prop.load( url.openStream() );     // Cargo propiedades desde InputStream de URL
      }
      catch (MalformedURLException e) {
	 mensajeError = new String("Mensaje de error: " + e.toString() );
      }
      catch (IOException e) {
	 mensajeError = new String("Mensaje de error: " + e.toString() );
      }
   }

   /***************************************************************************
    * Mtodo que recupera el valor de una clave del atributo 'prop'.
    * Si no la encuentra, devuelve el parmetro 'defecto'
    ***************************************************************************/
   public String getParametro(String clave, String defecto) {
      String retorno = defecto;
      try {
	 retorno = prop.getProperty(clave, defecto);
      }
      catch (Exception e) {
	 retorno = defecto;
      }
      finally {
	 return retorno;
      }
   }
   /****************************************************************************
    * Sobrecargado. Si no encuentra clave, devuelve ""
    ****************************************************************************/
   public String getParametro(String clave) {
      return getParametro(clave, "");
   }

   public String getDriver() {
      return getParametro( "basedatos.driver" );
   }
   public String getHost() {
      return getParametro( "basedatos.host" );
   }
}