/**
 * 
 */
package es.postigo.service;

import java.sql.ResultSetMetaData;
import java.util.ArrayList;
import java.util.HashMap;

import es.postigo.dto.MetadatoDTO;
import es.postigo.exceptions.ApplicationCoreException;

/**
 * @author informatica4
 *
 */
public interface TCService {

	/**
	 * Obtener hash map.
	 * 
	 * @param pStrElegidas
	 *            el string Elegidas
	 * @param pStrTabla
	 *            el string Tabla
	 * @param pStrWhere
	 *            el string Where
	 * @param pStrCondicion
	 *            el string Condicion
	 * @param pIntCursor
	 *            el integer Cursor
	 * @param pIntNumRegistros
	 *            el integer NumRegistros
	 * 
	 * @return un objeto HashMap
	 * 
	 * @throws ApplicationCoreException
	 *             the application exception
	 */
	public HashMap obtenerHashMap(String pStrElegidas, String pStrTabla, String pStrWhere, String pStrCondicion,
			int pIntCursor, int pIntNumRegistros) throws ApplicationCoreException;
	
	/**
	 * Obtener intercambio
	 * 
	 * @param pStrTabla
	 * 			el string Tabla
	 * 
	 * @param pStrElegidas
	 * 			el string Elegidas
	 * 
	 * @param pStrPrefijo
	 * 			Prefijo opcional para buscar las etiquetas en el properties
	 * 
	 * @return un objeto MetadatoDTO[]
	 * 
	 * @throws ApplicationCoreException
	 * 				the application exception
	 */
	public MetadatoDTO[] obtenerIntercambio(String pStrTabla, String pStrElegidas, String pStrPrefijo) throws ApplicationCoreException;

	/**
	 * Obtener hash orden.
	 * 
	 * @param pStrTabla
	 *            el string Tabla
	 * 
	 * @return un objeto HashMap
	 * 
	 * @throws ApplicationCoreException
	 *             the application exception
	 */
	public HashMap obtenerHashOrden(String pStrTabla) throws ApplicationCoreException;

	/**
	 * Obtener visibles no visibles.
	 * 
	 * @param pStrTabla
	 *            el string Tabla
	 * @param pStrVisibles
	 *            el string Visibles
	 * 
	 * @return un objeto ArrayList
	 * 
	 * @throws ApplicationCoreException
	 *             the application exception
	 */
	public ArrayList obtenerVisiblesNoVisibles(String pStrTabla, String pStrVisibles) throws ApplicationCoreException;

	/**
	 * Obtener numero registros.
	 * 
	 * @param pStrSQL
	 *            el string SQL
	 * 
	 * @return un objeto int
	 * 
	 * @throws ApplicationCoreException
	 *             the application exception
	 */
	public int obtenerNumeroRegistros(String pStrSQL) throws ApplicationCoreException;

	/**
	 * Obtener numero registros actual.
	 * 
	 * @param pStrTabla
	 *            el string Tabla
	 * @param pStrCondicion
	 *            el string Condicion
	 * 
	 * @return un objeto int
	 * 
	 * @throws ApplicationCoreException
	 *             the application exception
	 */
	public int obtenerNumeroRegistrosActual(String pStrTabla, String pStrCondicion) throws ApplicationCoreException;

	/**
	 * Obtiene datos.
	 * 
	 * @param pStrTabla
	 *            el string Tabla
	 * @param pStrColumnas
	 *            el string Columnas
	 * @param pStrWhere
	 *            el string Where
	 * @param pObjColumnas
	 *            el objeto Columnas
	 * 
	 * @return un objeto HashMap
	 * 
	 * @throws ApplicationCoreException
	 *             the application exception
	 */
	public HashMap obtieneDatos(String pStrTabla, String pStrColumnas, String pStrWhere, MetadatoDTO[] pObjColumnas)
			throws ApplicationCoreException;

	/**
	 * Calcular numero registros.
	 * 
	 * @param pStrSQL
	 *            el string SQL
	 * 
	 * @return un objeto long
	 * 
	 * @throws ApplicationCoreException
	 *             the application exception
	 */
	public long calcularNumeroRegistros(String pStrSQL) throws ApplicationCoreException;

	/**
	 * Hacer paginacion.
	 * 
	 * @param pLonTotal
	 *            el lon total
	 * @param pIntNumRegistros
	 *            el integer NumRegistros
	 * @param pIntCursor
	 *            el integer Cursor
	 * 
	 * @return un objeto ArrayList
	 * 
	 * @throws Exception
	 *             the exception
	 */
	public ArrayList hacerPaginacion(long pLonTotal, int pIntNumRegistros, int pIntCursor) throws Exception;

	/**
	 * Obtener vector paginacion.
	 * 
	 * @param pStrTabla
	 *            el string Tabla
	 * @param pStrWhere
	 *            el string Where
	 * @param pIntCursor
	 *            el integer Cursor
	 * @param pIntNumRegistros
	 *            el integer NumRegistros
	 * 
	 * @return un objeto ArrayList
	 * 
	 * @throws Exception
	 *             the exception
	 */
	public ArrayList obtenerVectorPaginacion(String pStrTabla, String pStrWhere, int pIntCursor, int pIntNumRegistros)
			throws Exception;

	/**
	 * Metodo que pasendole un ResultSetMetaData de una vista devuelve un
	 * objeto HashMap con los nombres de las columnas y sus porcentajes
	 * asociados.
	 * 
	 * @param pObjRsm
	 *            es un ResultSetMetaData de la vista 
	 * @return HashMap con los nombres de las columnas y sus porcentajes
	 *         asociados
	 * @exception ApplicationCoreException
	 *                si se produce alguna excepcion 
	 */
	public HashMap obtenerHashMap(ResultSetMetaData pObjRsm) throws ApplicationCoreException;

	/**
	 * Obtiene descripcion.
	 * 
	 * @param pStrSQL
	 *            el string SQL
	 * @param pIntNumColumnas
	 *            el integer NumColumnas
	 * 
	 * @return un objeto String
	 * 
	 * @throws Throwable
	 *             the throwable
	 */
	public String obtieneDescripcion(String pStrSQL, int pIntNumColumnas) throws Throwable;

	/***************************************************************************
	 * Obtiene un xml a partir de una lista de datos dependientes de otra tabla
	 * 
	 * @param pStrSQL
	 *            SQL que se va a ejecutar
	 * @return
	 * @throws Throwable
	 */
	public String obtieneComboDependiente(String pStrSQL) throws Throwable;
}
