/**
 * 
 */
package es.postigo.xml;

/**
 * @author informatica4
 *
 */

import java.util.ArrayList;

import es.postigo.exceptions.ArgumentNullException;

/**
 * Permite realizar validaciones contractuales de las propiedades de una clase.
 * 
 */
public class ContractualVerifier {

	/**
	 * Asegura que no es pasado un valor nulo como parmetro. 
	 * @param <T> Tipo del valor a comprobar.
	 * @param value Valor a comprobar.
	 */
	public static <T> void EnsureExistence(T value) {
		if (value == null)
			throw new ArgumentNullException("value");
	}
	
	/**
	 * Asegura que una cadena no es nula o vacia.
	 * @param value Cadena a comprobar.
	 */
	public static void EnsureNonEmptiness(String value) {
		ContractualVerifier.EnsureExistence(value);
		
		if (value.length() == 0)
			throw new IllegalArgumentException("La cadena de entrada debe tener al menos un carcter.");
	}
	
	/**
	 * Asegura que una cadena no es nula o vacia, incluyendo espacios en blanco.
	 * @param value Cadena a comprobar.
	 */
	public static void EnsureNonEmptinessStrict(String value) {
		ContractualVerifier.EnsureExistence(value);
		if (value.trim().length() == 0)
			throw new IllegalArgumentException("La cadena de entrada debe tener al menos un carcter que no sea un espacio.");
	}
	
	/**
	 * Asegura que una coleccin no es nula o vacia (contiene al menos un elemento).
	 * @param <T> Tipo de la coleccin a comprobar.
	 * @param value Coleccin a comprobar.
	 */
	public static <T> void EnsureNonEmptiness(ArrayList<T> value) {
		ContractualVerifier.EnsureExistence(value);
		
		if (value.isEmpty())
			throw new IllegalArgumentException("La coleccin debe tener al menos un elemento.");
	}
}
