/**
 * 
 */
package es.postigo.xml;

/**
 * @author informatica4
 *
 */

import java.io.Serializable;
import java.util.ArrayList;

/**
 * El Class ElementoXML.
 */
public class ElementoXML implements Serializable {

	/** La constante serialVersionUID. */
	private static final long serialVersionUID = 1L;
	
	/** El nombre. */
	private String nombre;
	
	/** El valor. */
	private String valor;
	
	/** El elementos. */
	private ArrayList<ElementoXML> elementos;
	
	/** El atributos. */
	private ArrayList<AtributoXML> atributos;
	
	/** El es root. */
	private boolean esRoot = false;

	/**
	 * Instancia un nuevo elemento xml.
	 * 
	 * @param pStrNombre
	 *            el string Nombre
	 */
	public ElementoXML(String pStrNombre) {
		
		ContractualVerifier.EnsureNonEmptinessStrict(pStrNombre);
		elementos = new ArrayList<ElementoXML>();
		atributos = new ArrayList<AtributoXML>();		
		this.nombre = pStrNombre;
	}

	/**
	 * Instancia un nuevo elemento xml.
	 * 
	 * @param pStrNombre
	 *            el string Nombre
	 * @param pStrValor
	 *            el string Valor
	 */
	public ElementoXML(String pStrNombre, String pStrValor) {
		
		this(pStrNombre);
		this.valor = pStrValor;
	}

	/**
	 * Obtiene el es root.
	 * 
	 * @return el es root
	 */
	public boolean getEsRoot() {
		
		return esRoot;
	}
	
	/**
	 * Asigna el es root.
	 * 
	 * @param esRoot
	 *            el nuevo es root
	 */
	public void setEsRoot(boolean esRoot) {
		
		this.esRoot = esRoot;
	}
	
	/**
	 * Obtiene el nombre.
	 * 
	 * @return el nombre
	 */
	public String getNombre() {
		
		return nombre;
	}

	/**
	 * Asigna el nombre.
	 * 
	 * @param nombre
	 *            el nuevo nombre
	 */
	public void setNombre(String nombre) {
		
		ContractualVerifier.EnsureNonEmptinessStrict(nombre);
		this.nombre = nombre;
	}

	/**
	 * Obtiene el valor.
	 * 
	 * @return el valor
	 */
	public String getValor() {
		
		return valor;
	}

	/**
	 * Asigna el valor.
	 * 
	 * @param valor
	 *            el nuevo valor
	 */
	public void setValor(String valor) {

		this.valor = valor;
	}

	/**
	 * Obtiene el elementos.
	 * 
	 * @return el elementos
	 */
	public ArrayList<ElementoXML> getElementos() {

		return elementos;
	}

	/**
	 * Asigna el elementos.
	 * 
	 * @param elementos
	 *            el nuevo elementos
	 */
	public void setElementos(ArrayList<ElementoXML> elementos) {

		this.elementos = elementos;
	}

	/**
	 * Obtiene el atributos.
	 * 
	 * @return el atributos
	 */
	public ArrayList<AtributoXML> getAtributos() {

		return atributos;
	}

	/**
	 * Asigna el atributos.
	 * 
	 * @param atributos
	 *            el nuevo atributos
	 */
	public void setAtributos(ArrayList<AtributoXML> atributos) {
		ContractualVerifier.EnsureExistence(atributos);
		this.atributos = atributos;
	}

	/**
	 * Aade el elemento.
	 * 
	 * @param pObjElemento
	 *            el objeto Elemento
	 */
	public void addElemento(ElementoXML pObjElemento) {
		ContractualVerifier.EnsureExistence(pObjElemento);
		elementos.add(pObjElemento);
	}

	/**
	 * Eliminar elemento.
	 * 
	 * @param pObjElemento
	 *            el objeto Elemento
	 */
	public void eliminarElemento(ElementoXML pObjElemento) {

		elementos.remove(pObjElemento);
	}

	/**
	 * Aade el atributo.
	 * 
	 * @param pObjAtributo
	 *            el objeto Atributo
	 */
	public void addAtributo(AtributoXML pObjAtributo) {
		ContractualVerifier.EnsureExistence(pObjAtributo);
		atributos.add(pObjAtributo);
	}

	/**
	 * Elimina el atributo.
	 * 
	 * @param pObjAtributoXML
	 *            el objeto AtributoXML
	 */
	public void removeAtributo(AtributoXML pObjAtributoXML) {

		atributos.remove(pObjAtributoXML);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		
		StringBuffer sb = new StringBuffer();
		sb.append("<");
		sb.append(nombre);
		
		if (!atributos.isEmpty()) {
			for (AtributoXML lObjAtributo : atributos) {
				sb.append(" ");
				sb.append(lObjAtributo.toString());
			}
		}
		
		if (!elementos.isEmpty())			
				sb.append(">\n");			
		else {
			if ((valor != null) || (esRoot))
				sb.append(">");
			else {
				//if (!esRoot) {
					sb.append(" />\n");
					return sb.toString();
				//}
			}			
		}
		
		if (valor != null) {
			sb.append(valor);
		}

		for (ElementoXML lObjElem : elementos)			
			sb.append(lObjElem.toString());

		sb.append("</");
		sb.append(nombre);
		sb.append(">\n");		

		return sb.toString();
	}

}